#!/usr/bin/python

# Copyright (c) 2021, Andreas Botzner <andreas at botzner dot com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: redis_data
short_description: Set key value pairs in Redis
version_added: 3.7.0
description:
  - Set key value pairs in Redis database.
author: "Andreas Botzner (@paginabianca)"
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  key:
    description:
      - Database key.
    required: true
    type: str
  value:
    description:
      - Value that key should be set to.
    type: str
  expiration:
    description:
      - Expiration time in milliseconds. Setting this option always results in a change in the database.
    type: int
  non_existing:
    description:
      - Only set key if it does not already exist.
    type: bool
  existing:
    description:
      - Only set key if it already exists.
    type: bool
  keep_ttl:
    description:
      - Retain the time to live associated with the key.
    type: bool
  state:
    description:
      - State of the key.
    default: present
    type: str
    choices:
      - present
      - absent

extends_documentation_fragment:
  - community.general._redis.documentation
  - community.general._attributes

seealso:
  - module: community.general.redis_data_incr
  - module: community.general.redis_data_info
  - module: community.general.redis
"""

EXAMPLES = r"""
- name: Set key foo=bar on localhost with no username
  community.general.redis_data:
    login_host: localhost
    login_password: supersecret
    key: foo
    value: bar
    state: present

- name: Set key foo=bar if non existing with expiration of 30s
  community.general.redis_data:
    login_host: localhost
    login_password: supersecret
    key: foo
    value: bar
    non_existing: true
    expiration: 30000
    state: present

- name: Set key foo=bar if existing and keep current TTL
  community.general.redis_data:
    login_host: localhost
    login_password: supersecret
    key: foo
    value: bar
    existing: true
    keep_ttl: true

- name: Set key foo=bar on redishost with custom ca-cert file
  community.general.redis_data:
    login_host: redishost
    login_password: supersecret
    login_user: someuser
    validate_certs: true
    ssl_ca_certs: /path/to/ca/certs
    key: foo
    value: bar

- name: Delete key foo on localhost with no username
  community.general.redis_data:
    login_host: localhost
    login_password: supersecret
    key: foo
    state: absent
"""

RETURN = r"""
old_value:
  description: Value of key before setting.
  returned: on_success if O(state=present) and key exists in database.
  type: str
  sample: 'old_value_of_key'
value:
  description: Value key was set to.
  returned: on success if O(state=present).
  type: str
  sample: 'new_value_of_key'
msg:
  description: A short message.
  returned: always
  type: str
  sample: 'Set key: foo to bar'
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._redis import (
    RedisAnsible,
    fail_imports,
    redis_auth_argument_spec,
)


def main():
    redis_auth_args = redis_auth_argument_spec()
    module_args = dict(
        key=dict(type="str", required=True, no_log=False),
        value=dict(type="str"),
        expiration=dict(type="int"),
        non_existing=dict(type="bool"),
        existing=dict(type="bool"),
        keep_ttl=dict(type="bool"),
        state=dict(type="str", default="present", choices=["present", "absent"]),
    )
    module_args.update(redis_auth_args)

    module = AnsibleModule(
        argument_spec=module_args,
        supports_check_mode=True,
        required_if=[("state", "present", ("value",))],
        mutually_exclusive=[["non_existing", "existing"], ["keep_ttl", "expiration"]],
    )
    fail_imports(module)

    redis = RedisAnsible(module)

    key = module.params["key"]
    value = module.params["value"]
    px = module.params["expiration"]
    nx = module.params["non_existing"]
    xx = module.params["existing"]
    keepttl = module.params["keep_ttl"]
    state = module.params["state"]
    set_args = {"name": key, "value": value, "px": px, "nx": nx, "xx": xx, "keepttl": keepttl}

    result = {"changed": False}

    old_value = None
    try:
        old_value = redis.connection.get(key)
    except Exception as e:
        msg = f"Failed to get value of key: {key} with exception: {e}"
        result["msg"] = msg
        module.fail_json(**result)

    if state == "absent":
        if module.check_mode:
            if old_value is None:
                msg = f"Key: {key} not present"
                result["msg"] = msg
                module.exit_json(**result)
            else:
                msg = f"Deleted key: {key}"
                result["msg"] = msg
                module.exit_json(**result)
        try:
            ret = redis.connection.delete(key)
            if ret == 0:
                msg = f"Key: {key} not present"
                result["msg"] = msg
                module.exit_json(**result)
            else:
                msg = f"Deleted key: {key}"
                result["msg"] = msg
                result["changed"] = True
                module.exit_json(**result)
        except Exception as e:
            msg = f"Failed to delete key: {key} with exception: {e}"
            result["msg"] = msg
            module.fail_json(**result)

    old_value = None
    try:
        old_value = redis.connection.get(key)
    except Exception as e:
        msg = f"Failed to get value of key: {key} with exception: {e}"
        result["msg"] = msg
        module.fail_json(**result)

    result["old_value"] = old_value
    if old_value == value and keepttl is not False and px is None:
        msg = f"Key {key} already has desired value"
        result["msg"] = msg
        result["value"] = value
        module.exit_json(**result)
    if module.check_mode:
        result["msg"] = f"Set key: {key}"
        result["value"] = value
        module.exit_json(**result)
    try:
        ret = redis.connection.set(**set_args)
        if ret is None:
            if nx:
                msg = f"Could not set key: {key}. Key already present."
            else:
                msg = f"Could not set key: {key}. Key not present."
            result["msg"] = msg
            module.fail_json(**result)
        msg = f"Set key: {key}"
        result["msg"] = msg
        result["changed"] = True
        result["value"] = value
        module.exit_json(**result)
    except Exception as e:
        msg = f"Failed to set key: {key} with exception: {e}"
        result["msg"] = msg
        module.fail_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2022 Western Digital Corporation
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: wdc_redfish_command
short_description: Manages WDC UltraStar Data102 Out-Of-Band controllers using Redfish APIs
version_added: 5.4.0
description:
  - Builds Redfish URIs locally and sends them to remote OOB controllers to perform an action.
  - Manages OOB controller firmware. For example, Firmware Activate, Update and Activate.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._redfish
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  category:
    required: true
    description:
      - Category to execute on OOB controller.
    type: str
  command:
    required: true
    description:
      - List of commands to execute on OOB controller.
    type: list
    elements: str
  baseuri:
    description:
      - Base URI of OOB controller. Must include this or O(ioms).
    type: str
  ioms:
    description:
      - List of IOM FQDNs for the enclosure. Must include this or O(baseuri).
    type: list
    elements: str
  username:
    description:
      - User for authentication with OOB controller.
    type: str
  password:
    description:
      - Password for authentication with OOB controller.
    type: str
  auth_token:
    description:
      - Security token for authentication with OOB controller.
    type: str
  timeout:
    description:
      - Timeout in seconds for URL requests to OOB controller.
    default: 10
    type: int
  resource_id:
    description:
      - ID of the component to modify, such as V(Enclosure), V(IOModuleAFRU), V(PowerSupplyBFRU), V(FanExternalFRU3), or V(FanInternalFRU).
    type: str
    version_added: 5.4.0
  update_image_uri:
    description:
      - The URI of the image for the update.
    type: str
  update_creds:
    description:
      - The credentials for retrieving the update image.
    type: dict
    suboptions:
      username:
        description:
          - The username for retrieving the update image.
        type: str
      password:
        description:
          - The password for retrieving the update image.
        type: str
  validate_certs:
    version_added: 10.6.0
  ca_path:
    version_added: 10.6.0
  ciphers:
    version_added: 10.6.0
notes:
  - In the inventory, you can specify baseuri or ioms. See the EXAMPLES section.
  - Ioms is a list of FQDNs for the enclosure's IOMs.
author: Mike Moerk (@mikemoerk)
"""

EXAMPLES = r"""
- name: Firmware Activate (required after SimpleUpdate to apply the new firmware)
  community.general.wdc_redfish_command:
    category: Update
    command: FWActivate
    ioms: "{{ ioms }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Firmware Activate with individual IOMs specified
  community.general.wdc_redfish_command:
    category: Update
    command: FWActivate
    ioms:
      - iom1.wdc.com
      - iom2.wdc.com
    username: "{{ username }}"
    password: "{{ password }}"

- name: Firmware Activate with baseuri specified
  community.general.wdc_redfish_command:
    category: Update
    command: FWActivate
    baseuri: "iom1.wdc.com"
    username: "{{ username }}"
    password: "{{ password }}"


- name: Update and Activate (orchestrates firmware update and activation with a single command)
  community.general.wdc_redfish_command:
    category: Update
    command: UpdateAndActivate
    ioms: "{{ ioms }}"
    username: "{{ username }}"
    password: "{{ password }}"
    update_image_uri: "{{ update_image_uri }}"
    update_creds:
      username: operator
      password: supersecretpwd

- name: Turn on enclosure indicator LED
  community.general.wdc_redfish_command:
    category: Chassis
    resource_id: Enclosure
    command: IndicatorLedOn
    username: "{{ username }}"
    password: "{{ password }}"

- name: Turn off IOM A indicator LED
  community.general.wdc_redfish_command:
    category: Chassis
    resource_id: IOModuleAFRU
    command: IndicatorLedOff
    username: "{{ username }}"
    password: "{{ password }}"

- name: Turn on Power Supply B indicator LED
  community.general.wdc_redfish_command:
    category: Chassis
    resource_id: PowerSupplyBFRU
    command: IndicatorLedOn
    username: "{{ username }}"
    password: "{{ password }}"

- name: Turn on External Fan 3 indicator LED
  community.general.wdc_redfish_command:
    category: Chassis
    resource_id: FanExternalFRU3
    command: IndicatorLedOn
    username: "{{ username }}"
    password: "{{ password }}"

- name: Turn on Internal Fan indicator LED
  community.general.wdc_redfish_command:
    category: Chassis
    resource_id: FanInternalFRU
    command: IndicatorLedOn
    username: "{{ username }}"
    password: "{{ password }}"

- name: Set chassis to Low Power Mode
  community.general.wdc_redfish_command:
    category: Chassis
    resource_id: Enclosure
    command: PowerModeLow

- name: Set chassis to Normal Power Mode
  community.general.wdc_redfish_command:
    category: Chassis
    resource_id: Enclosure
    command: PowerModeNormal
"""

RETURN = r"""
msg:
  description: Message with action result or error description.
  returned: always
  type: str
  sample: "Action was successful"
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.general.plugins.module_utils._redfish_utils import REDFISH_COMMON_ARGUMENT_SPEC
from ansible_collections.community.general.plugins.module_utils._wdc_redfish_utils import WdcRedfishUtils

CATEGORY_COMMANDS_ALL = {
    "Update": ["FWActivate", "UpdateAndActivate"],
    "Chassis": [
        "IndicatorLedOn",
        "IndicatorLedOff",
        "PowerModeLow",
        "PowerModeNormal",
    ],
}


def main():
    argument_spec = dict(
        category=dict(required=True),
        command=dict(required=True, type="list", elements="str"),
        ioms=dict(type="list", elements="str"),
        baseuri=dict(),
        username=dict(),
        password=dict(no_log=True),
        auth_token=dict(no_log=True),
        update_creds=dict(type="dict", options=dict(username=dict(), password=dict(no_log=True))),
        resource_id=dict(),
        update_image_uri=dict(),
        timeout=dict(type="int", default=10),
    )
    argument_spec.update(REDFISH_COMMON_ARGUMENT_SPEC)
    module = AnsibleModule(
        argument_spec,
        required_together=[
            ("username", "password"),
        ],
        required_one_of=[("username", "auth_token"), ("baseuri", "ioms")],
        mutually_exclusive=[
            ("username", "auth_token"),
        ],
        supports_check_mode=True,
    )

    category = module.params["category"]
    command_list = module.params["command"]

    # admin credentials used for authentication
    creds = {"user": module.params["username"], "pswd": module.params["password"], "token": module.params["auth_token"]}

    # timeout
    timeout = module.params["timeout"]

    # Resource to modify
    resource_id = module.params["resource_id"]

    # Check that Category is valid
    if category not in CATEGORY_COMMANDS_ALL:
        module.fail_json(
            msg=f"Invalid Category '{category}'. Valid Categories = {sorted(CATEGORY_COMMANDS_ALL.keys())}"
        )

    # Check that all commands are valid
    for cmd in command_list:
        # Fail if even one command given is invalid
        if cmd not in CATEGORY_COMMANDS_ALL[category]:
            module.fail_json(msg=f"Invalid Command '{cmd}'. Valid Commands = {CATEGORY_COMMANDS_ALL[category]}")

    # Build root URI(s)
    if module.params.get("baseuri") is not None:
        root_uris = [f"https://{module.params['baseuri']}"]
    else:
        root_uris = [f"https://{iom}" for iom in module.params["ioms"]]
    rf_utils = WdcRedfishUtils(creds, root_uris, timeout, module, resource_id=resource_id, data_modification=True)

    # Organize by Categories / Commands

    if category == "Update":
        # execute only if we find UpdateService resources
        resource = rf_utils._find_updateservice_resource()
        if resource["ret"] is False:
            module.fail_json(msg=resource["msg"])
        # update options
        update_opts = {"update_creds": module.params["update_creds"]}
        for command in command_list:
            if command == "FWActivate":
                if module.check_mode:
                    result = {"ret": True, "changed": True, "msg": "FWActivate not performed in check mode."}
                else:
                    result = rf_utils.firmware_activate(update_opts)
            elif command == "UpdateAndActivate":
                update_opts["update_image_uri"] = module.params["update_image_uri"]
                result = rf_utils.update_and_activate(update_opts)

    elif category == "Chassis":
        result = rf_utils._find_chassis_resource()
        if result["ret"] is False:
            module.fail_json(msg=result["msg"])

        led_commands = ["IndicatorLedOn", "IndicatorLedOff"]

        # Check if more than one led_command is present
        num_led_commands = sum([command in led_commands for command in command_list])
        if num_led_commands > 1:
            result = {"ret": False, "msg": "Only one IndicatorLed command should be sent at a time."}
        else:
            for command in command_list:
                if command.startswith("IndicatorLed"):
                    result = rf_utils.manage_chassis_indicator_led(command)
                elif command.startswith("PowerMode"):
                    result = rf_utils.manage_chassis_power_mode(command)

    if result["ret"] is False:
        module.fail_json(msg=to_native(result["msg"]))
    else:
        del result["ret"]
        changed = result.get("changed", True)
        session = result.get("session", dict())
        module.exit_json(
            changed=changed,
            session=session,
            msg="Action was successful"
            if not module.check_mode
            else result.get("msg", "No action performed in check mode."),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, Ryan Scott Brown <ryansb@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: terraform
short_description: Manages a Terraform deployment (and plans)
description:
  - Provides support for deploying resources with Terraform and pulling resource information back into Ansible.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
    version_added: 8.3.0
options:
  state:
    choices: ['planned', 'present', 'absent']
    description:
      - Goal state of given stage/project.
    type: str
    default: present
  binary_path:
    description:
      - The path of a C(terraform) binary to use, relative to the 'service_path' unless you supply an absolute path.
    type: path
  project_path:
    description:
      - The path to the root of the Terraform directory with the C(vars.tf)/C(main.tf)/etc to use.
    type: path
    required: true
  plugin_paths:
    description:
      - List of paths containing Terraform plugin executable files.
      - Plugin executables can be downloaded from U(https://releases.hashicorp.com/).
      - When set, the plugin discovery and auto-download behavior of Terraform is disabled.
      - The directory structure in the plugin path can be tricky. The Terraform docs
        U(https://learn.hashicorp.com/tutorials/terraform/automate-terraform#pre-installed-plugins)
        show a simple directory of files, but actually, the directory structure has to follow the same structure you would
        see if Terraform auto-downloaded the plugins. See the examples below for a tree output of an example plugin directory.
    type: list
    elements: path
    version_added: 3.0.0
  workspace:
    description:
      - The terraform workspace to work with. This sets the E(TF_WORKSPACE) environmental variable that is used to override
        workspace selection. For more information about workspaces have a look at U(https://developer.hashicorp.com/terraform/language/state/workspaces).
    type: str
    default: default
  purge_workspace:
    description:
      - Only works with state = absent.
      - If V(true), the O(workspace) is deleted after the C(terraform destroy) action.
      - If O(workspace=default) then it is not deleted.
    default: false
    type: bool
  plan_file:
    description:
      - The path to an existing Terraform plan file to apply. If this is not specified, Ansible builds a new TF plan and execute
        it. Note that this option is required if O(state=planned).
    type: path
  state_file:
    description:
      - The path to an existing Terraform state file to use when building plan. If this is not specified, the default C(terraform.tfstate)
        is used.
      - This option is ignored when plan is specified.
    type: path
  variables_files:
    description:
      - The path to a variables file for Terraform to fill into the TF configurations. This can accept a list of paths to
        multiple variables files.
    type: list
    elements: path
    aliases: ['variables_file']
  variables:
    description:
      - A group of key-values pairs to override template variables or those in variables files. By default, only string and
        number values are allowed, which are passed on unquoted.
      - Support complex variable structures (lists, dictionaries, numbers, and booleans) to reflect terraform variable syntax
        when O(complex_vars=true).
      - Ansible integers or floats are mapped to terraform numbers.
      - Ansible strings are mapped to terraform strings.
      - Ansible dictionaries are mapped to terraform objects.
      - Ansible lists are mapped to terraform lists.
      - Ansible booleans are mapped to terraform booleans.
      - B(Note) passwords passed as variables are visible in the log output. Make sure to use C(no_log=true) in production!.
    type: dict
  complex_vars:
    description:
      - Enable/disable capability to handle complex variable structures for C(terraform).
      - If V(true) the O(variables) also accepts dictionaries, lists, and booleans to be passed to C(terraform). Strings that
        are passed are correctly quoted.
      - When disabled, supports only simple variables (strings, integers, and floats), and passes them on unquoted.
    type: bool
    default: false
    version_added: 5.7.0
  targets:
    description:
      - A list of specific resources to target in this plan/application. The resources selected here are also auto-include
        any dependencies.
    type: list
    elements: str
    default: []
  lock:
    description:
      - Enable statefile locking, if you use a service that accepts locks (such as S3+DynamoDB) to store your statefile.
    type: bool
    default: true
  lock_timeout:
    description:
      - How long to maintain the lock on the statefile, if you use a service that accepts locks (such as S3+DynamoDB).
    type: int
  force_init:
    description:
      - To avoid duplicating infra, if a state file cannot be found this forces a C(terraform init). Generally, this should
        be turned off unless you intend to provision an entirely new Terraform deployment.
    default: false
    type: bool
  overwrite_init:
    description:
      - Run init even if C(.terraform/terraform.tfstate) already exists in O(project_path).
    default: true
    type: bool
    version_added: '3.2.0'
  backend_config:
    description:
      - A group of key-values to provide at init stage to the -backend-config parameter.
    type: dict
  backend_config_files:
    description:
      - The path to a configuration file to provide at init state to the -backend-config parameter. This can accept a list
        of paths to multiple configuration files.
    type: list
    elements: path
    version_added: '0.2.0'
  provider_upgrade:
    description:
      - Allows Terraform init to upgrade providers to versions specified in the project's version constraints.
    default: false
    type: bool
    version_added: 4.8.0
  init_reconfigure:
    description:
      - Forces backend reconfiguration during init.
    default: false
    type: bool
    version_added: '1.3.0'
  check_destroy:
    description:
      - Apply only when no resources are destroyed. Note that this only prevents "destroy" actions, but not "destroy and re-create"
        actions. This option is ignored when O(state=absent).
    type: bool
    default: false
    version_added: '3.3.0'
  parallelism:
    description:
      - Restrict concurrent operations when Terraform applies the plan.
    type: int
    version_added: '3.8.0'
  no_color:
    description:
      - If V(true), suppress color codes in output from Terraform commands.
      - If V(false), allows Terraform to use color codes in its output.
    type: bool
    default: true
    version_added: 11.0.0
notes:
  - To just run a C(terraform plan), use check mode.
requirements: ["terraform"]
author: "Ryan Scott Brown (@ryansb)"
"""

EXAMPLES = r"""
- name: Basic deploy of a service
  community.general.terraform:
    project_path: '{{ project_dir }}'
    state: present

- name: Deploy with color output enabled
  community.general.terraform:
    project_path: '{{ project_dir }}'
    state: present
    no_color: false

- name: Define the backend configuration at init
  community.general.terraform:
    project_path: 'project/'
    state: "{{ state }}"
    force_init: true
    backend_config:
      region: "eu-west-1"
      bucket: "some-bucket"
      key: "random.tfstate"

- name: Define the backend configuration with one or more files at init
  community.general.terraform:
    project_path: 'project/'
    state: "{{ state }}"
    force_init: true
    backend_config_files:
      - /path/to/backend_config_file_1
      - /path/to/backend_config_file_2

- name: Disable plugin discovery and auto-download by setting plugin_paths
  community.general.terraform:
    project_path: 'project/'
    state: "{{ state }}"
    force_init: true
    plugin_paths:
      - /path/to/plugins_dir_1
      - /path/to/plugins_dir_2

- name: Complex variables example
  community.general.terraform:
    project_path: '{{ project_dir }}'
    state: present
    complex_vars: true
    variables:
      vm_name: "{{ inventory_hostname }}"
      vm_vcpus: 2
      vm_mem: 2048
      vm_additional_disks:
        - label: "Third Disk"
          size: 40
          thin_provisioned: true
          unit_number: 2
        - label: "Fourth Disk"
          size: 22
          thin_provisioned: true
          unit_number: 3
    force_init: true

### Example directory structure for plugin_paths example
# $ tree /path/to/plugins_dir_1
# /path/to/plugins_dir_1/
# └── registry.terraform.io
#     └── hashicorp
#         └── vsphere
#             ├── 1.24.0
#             │   └── linux_amd64
#             │       └── terraform-provider-vsphere_v1.24.0_x4
#             └── 1.26.0
#                 └── linux_amd64
#                     └── terraform-provider-vsphere_v1.26.0_x4
"""

RETURN = r"""
outputs:
  type: complex
  description: A dictionary of all the TF outputs by their assigned name. Use RV(ignore:outputs.MyOutputName.value) to access
    the value.
  returned: on success
  sample: '{"bukkit_arn": {"sensitive": false, "type": "string", "value": "arn:aws:s3:::tf-test-bukkit"}'
  contains:
    sensitive:
      type: bool
      returned: always
      description: Whether Terraform has marked this value as sensitive.
    type:
      type: str
      returned: always
      description: The type of the value (string, int, and so on).
    value:
      type: str
      returned: always
      description: The value of the output as interpolated by Terraform.
command:
  type: str
  description: Full C(terraform) command built by this module, in case you want to re-run the command outside the module or
    debug a problem.
  returned: always
  sample: terraform apply ...
"""

import json
import os
import tempfile
from shlex import quote as shlex_quote

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._version import LooseVersion

module = None


def get_version(bin_path):
    extract_version = module.run_command([bin_path, "version", "-json"])
    terraform_version = (json.loads(extract_version[1]))["terraform_version"]
    return terraform_version


def preflight_validation(bin_path, project_path, version, variables_args=None, plan_file=None, no_color=True):
    if project_path is None or "/" not in project_path:
        module.fail_json(msg="Path for Terraform project can not be None or ''.")
    if not os.path.exists(bin_path):
        module.fail_json(
            msg=f"Path for Terraform binary '{bin_path}' doesn't exist on this host - check the path and try again please."
        )
    if not os.path.isdir(project_path):
        module.fail_json(
            msg=f"Path for Terraform project '{project_path}' doesn't exist on this host - check the path and try again please."
        )
    cmd = [bin_path, "validate"]
    if no_color:
        cmd.append("-no-color")
    if LooseVersion(version) < LooseVersion("0.15.0"):
        module.run_command(cmd + variables_args, check_rc=True, cwd=project_path)
    else:
        module.run_command(cmd, check_rc=True, cwd=project_path)


def _state_args(state_file):
    if not state_file:
        return []
    if not os.path.exists(state_file):
        module.warn(
            f'Could not find state_file "{state_file}", the process will not destroy any resources, please check your state file path.'
        )
    return ["-state", state_file]


def init_plugins(
    bin_path,
    project_path,
    backend_config,
    backend_config_files,
    init_reconfigure,
    provider_upgrade,
    plugin_paths,
    workspace,
    no_color=True,
):
    command = [bin_path, "init", "-input=false"]
    if no_color:
        command.append("-no-color")
    if backend_config:
        for key, val in backend_config.items():
            command.extend(["-backend-config", f"{key}={val}"])
    if backend_config_files:
        for f in backend_config_files:
            command.extend(["-backend-config", f])
    if init_reconfigure:
        command.extend(["-reconfigure"])
    if provider_upgrade:
        command.extend(["-upgrade"])
    if plugin_paths:
        for plugin_path in plugin_paths:
            command.extend(["-plugin-dir", plugin_path])
    rc, out, err = module.run_command(
        command, check_rc=True, cwd=project_path, environ_update={"TF_WORKSPACE": workspace}
    )


def get_workspace_context(bin_path, project_path, no_color=True):
    workspace_ctx = {"current": "default", "all": []}
    command = [bin_path, "workspace", "list"]
    if no_color:
        command.append("-no-color")

    rc, out, err = module.run_command(command, cwd=project_path)
    if rc != 0:
        module.warn(f"Failed to list Terraform workspaces:\n{err}")
    for item in out.split("\n"):
        stripped_item = item.strip()
        if not stripped_item:
            continue
        elif stripped_item.startswith("* "):
            workspace_ctx["current"] = stripped_item.replace("* ", "")
            workspace_ctx["all"].append(stripped_item.replace("* ", ""))
        else:
            workspace_ctx["all"].append(stripped_item)
    return workspace_ctx


def _workspace_cmd(bin_path, project_path, action, workspace, no_color=True):
    command = [bin_path, "workspace", action, workspace]
    if no_color:
        command.append("-no-color")
    rc, out, err = module.run_command(command, check_rc=True, cwd=project_path)
    return rc, out, err


def create_workspace(bin_path, project_path, workspace, no_color=True):
    _workspace_cmd(bin_path, project_path, "new", workspace, no_color)


def select_workspace(bin_path, project_path, workspace, no_color=True):
    _workspace_cmd(bin_path, project_path, "select", workspace, no_color)


def remove_workspace(bin_path, project_path, workspace, no_color=True):
    _workspace_cmd(bin_path, project_path, "delete", workspace, no_color)


def build_plan(command, project_path, variables_args, state_file, targets, state, args, plan_path=None, no_color=True):
    if plan_path is None:
        f, plan_path = tempfile.mkstemp(suffix=".tfplan")

    local_command = command[:]

    plan_command = [command[0], "plan"]

    if state == "planned":
        for c in local_command[1:]:
            plan_command.append(c)

    if state == "present":
        for a in args:
            local_command.remove(a)
        for c in local_command[1:]:
            plan_command.append(c)

    if state == "absent":
        for a in args:
            plan_command.append(a)

    plan_options = ["-input=false", "-detailed-exitcode", "-out", plan_path]
    if no_color:
        plan_options.insert(0, "-no-color")
    plan_command.extend(plan_options)

    for t in targets:
        plan_command.extend(["-target", t])

    plan_command.extend(_state_args(state_file))

    rc, out, err = module.run_command(plan_command + variables_args, cwd=project_path)

    if rc == 0:
        # no changes
        return plan_path, False, out, err, plan_command if state == "planned" else command
    elif rc == 1:
        # failure to plan
        module.fail_json(
            msg=(
                f"Terraform plan could not be created\nSTDOUT: {out}\nSTDERR: {err}\n"
                f"COMMAND: {' '.join(plan_command)} {' '.join([shlex_quote(arg) for arg in variables_args])}"
            )
        )
    elif rc == 2:
        # changes, but successful
        return plan_path, True, out, err, plan_command if state == "planned" else command

    module.fail_json(
        msg=f"Terraform plan failed with unexpected exit code {rc}.\nSTDOUT: {out}\nSTDERR: {err}\n"
        f"COMMAND: {' '.join(plan_command)} {' '.join([shlex_quote(arg) for arg in variables_args])}"
    )


def get_diff(diff_output):
    def get_tf_resource_address(e):
        return e["resource"]

    diff_json_output = json.loads(diff_output)

    # Ignore diff if resource_changes does not exists in tfplan
    if "resource_changes" in diff_json_output:
        tf_reosource_changes = diff_json_output["resource_changes"]
    else:
        module.warn("Cannot find resource_changes in terraform plan, diff/check ignored")
        return False, {}

    diff_after = []
    diff_before = []
    changed = False
    for item in tf_reosource_changes:
        item_change = item["change"]
        tf_before_state = {"resource": item["address"], "change": item["change"]["before"]}
        tf_after_state = {"resource": item["address"], "change": item["change"]["after"]}

        if item_change["actions"] == ["update"] or item_change["actions"] == ["delete", "create"]:
            diff_before.append(tf_before_state)
            diff_after.append(tf_after_state)
            changed = True

        if item_change["actions"] == ["delete"]:
            diff_before.append(tf_before_state)
            changed = True

        if item_change["actions"] == ["create"]:
            diff_after.append(tf_after_state)
            changed = True

    diff_before.sort(key=get_tf_resource_address)
    diff_after.sort(key=get_tf_resource_address)

    return changed, dict(
        before=({"data": diff_before}),
        after=({"data": diff_after}),
    )


def main():
    global module
    module = AnsibleModule(
        argument_spec=dict(
            project_path=dict(required=True, type="path"),
            binary_path=dict(type="path"),
            plugin_paths=dict(type="list", elements="path"),
            workspace=dict(type="str", default="default"),
            purge_workspace=dict(type="bool", default=False),
            state=dict(default="present", choices=["present", "absent", "planned"]),
            variables=dict(type="dict"),
            complex_vars=dict(type="bool", default=False),
            variables_files=dict(aliases=["variables_file"], type="list", elements="path"),
            plan_file=dict(type="path"),
            state_file=dict(type="path"),
            targets=dict(type="list", elements="str", default=[]),
            lock=dict(type="bool", default=True),
            lock_timeout=dict(type="int"),
            force_init=dict(type="bool", default=False),
            backend_config=dict(type="dict"),
            backend_config_files=dict(type="list", elements="path"),
            init_reconfigure=dict(type="bool", default=False),
            overwrite_init=dict(type="bool", default=True),
            check_destroy=dict(type="bool", default=False),
            parallelism=dict(type="int"),
            provider_upgrade=dict(type="bool", default=False),
            no_color=dict(type="bool", default=True),
        ),
        required_if=[("state", "planned", ["plan_file"])],
        supports_check_mode=True,
    )

    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}

    project_path = module.params.get("project_path")
    bin_path = module.params.get("binary_path")
    plugin_paths = module.params.get("plugin_paths")
    workspace = module.params.get("workspace")
    purge_workspace = module.params.get("purge_workspace")
    state = module.params.get("state")
    variables = module.params.get("variables") or {}
    complex_vars = module.params.get("complex_vars")
    variables_files = module.params.get("variables_files")
    plan_file = module.params.get("plan_file")
    state_file = module.params.get("state_file")
    force_init = module.params.get("force_init")
    backend_config = module.params.get("backend_config")
    backend_config_files = module.params.get("backend_config_files")
    init_reconfigure = module.params.get("init_reconfigure")
    overwrite_init = module.params.get("overwrite_init")
    check_destroy = module.params.get("check_destroy")
    provider_upgrade = module.params.get("provider_upgrade")
    no_color = module.params.get("no_color")

    if bin_path is not None:
        command = [bin_path]
    else:
        command = [module.get_bin_path("terraform", required=True)]

    checked_version = get_version(command[0])

    if LooseVersion(checked_version) < LooseVersion("0.15.0"):
        if no_color:
            DESTROY_ARGS = ("destroy", "-no-color", "-force")
            APPLY_ARGS = ("apply", "-no-color", "-input=false", "-auto-approve=true")
        else:
            DESTROY_ARGS = ("destroy", "-force")
            APPLY_ARGS = ("apply", "-input=false", "-auto-approve=true")
    else:
        if no_color:
            DESTROY_ARGS = ("destroy", "-no-color", "-auto-approve")
            APPLY_ARGS = ("apply", "-no-color", "-input=false", "-auto-approve")
        else:
            DESTROY_ARGS = ("destroy", "-auto-approve")
            APPLY_ARGS = ("apply", "-input=false", "-auto-approve")

    if force_init:
        if overwrite_init or not os.path.isfile(os.path.join(project_path, ".terraform", "terraform.tfstate")):
            init_plugins(
                command[0],
                project_path,
                backend_config,
                backend_config_files,
                init_reconfigure,
                provider_upgrade,
                plugin_paths,
                workspace,
                no_color,
            )

    workspace_ctx = get_workspace_context(command[0], project_path, no_color)
    if workspace_ctx["current"] != workspace:
        if workspace not in workspace_ctx["all"]:
            create_workspace(command[0], project_path, workspace, no_color)
        else:
            select_workspace(command[0], project_path, workspace, no_color)

    if state == "present":
        command.extend(APPLY_ARGS)
    elif state == "absent":
        command.extend(DESTROY_ARGS)

    if state == "present" and module.params.get("parallelism") is not None:
        command.append(f"-parallelism={module.params.get('parallelism')}")

    def format_args(vars):
        if vars is None:
            return "null"
        elif isinstance(vars, str):
            escaped = vars.replace("\\", "\\\\").replace('"', '\\"')
            return f'"{escaped}"'.replace("\n", "\\n")
        elif isinstance(vars, bool):
            if vars:
                return "true"
            else:
                return "false"
        return str(vars)

    def process_complex_args(vars):
        ret_out = []
        if isinstance(vars, dict):
            for k, v in vars.items():
                if isinstance(v, dict):
                    ret_out.append(f"{k}={{{process_complex_args(v)}}}")
                elif isinstance(v, list):
                    ret_out.append(f"{k}={process_complex_args(v)}")
                elif isinstance(v, (int, float, str, bool)) or v is None:
                    ret_out.append(f"{k}={format_args(v)}")
                else:
                    # only to handle anything unforeseen
                    module.fail_json(
                        msg="Supported types are, dictionaries, lists, strings, integers, boolean and float."
                    )
        if isinstance(vars, list):
            l_out = []
            for item in vars:
                if isinstance(item, dict):
                    l_out.append(f"{{{process_complex_args(item)}}}")
                elif isinstance(item, list):
                    l_out.append(f"{process_complex_args(item)}")
                elif isinstance(item, (str, int, float, bool)) or item is None:
                    l_out.append(format_args(item))
                else:
                    # only to handle anything unforeseen
                    module.fail_json(
                        msg="Supported types are, dictionaries, lists, strings, integers, boolean and float."
                    )

            ret_out.append(f"[{','.join(l_out)}]")
        return ",".join(ret_out)

    variables_args = []
    if complex_vars:
        for k, v in variables.items():
            if isinstance(v, dict):
                variables_args.extend(["-var", f"{k}={{{process_complex_args(v)}}}"])
            elif isinstance(v, list):
                variables_args.extend(["-var", f"{k}={process_complex_args(v)}"])
            # on the top-level we need to pass just the python string with necessary
            # terraform string escape sequences
            elif isinstance(v, str):
                variables_args.extend(["-var", f"{k}={v}"])
            else:
                variables_args.extend(["-var", f"{k}={format_args(v)}"])
    else:
        for k, v in variables.items():
            variables_args.extend(["-var", f"{k}={v}"])

    if variables_files:
        for f in variables_files:
            variables_args.extend(["-var-file", f])

    preflight_validation(command[0], project_path, checked_version, variables_args, plan_file, no_color)

    if module.params.get("lock") is not None:
        if module.params.get("lock"):
            command.append("-lock=true")
        else:
            command.append("-lock=false")
    if module.params.get("lock_timeout") is not None:
        command.append(f"-lock-timeout={module.params.get('lock_timeout')}s")

    for t in module.params.get("targets") or []:
        command.extend(["-target", t])

    # we aren't sure if this plan will result in changes, so assume yes
    needs_application, changed = True, False

    out, err = "", ""

    if state == "absent":
        command.extend(variables_args)
    elif state == "present" and plan_file:
        if any([os.path.isfile(f"{project_path}/{plan_file}"), os.path.isfile(plan_file)]):
            command.append(plan_file)
        else:
            module.fail_json(msg=f'Could not find plan_file "{plan_file}", check the path and try again.')
    else:
        plan_file, needs_application, out, err, command = build_plan(
            command,
            project_path,
            variables_args,
            state_file,
            module.params.get("targets"),
            state,
            APPLY_ARGS,
            plan_file,
            no_color,
        )
        if state == "present" and check_destroy and "- destroy" in out:
            module.fail_json(
                msg="Aborting command because it would destroy some resources. "
                "Consider switching the 'check_destroy' to false to suppress this error"
            )
        command.append(plan_file)

    result_diff = dict()
    if module._diff or module.check_mode:
        if state == "absent":
            plan_absent_args = ["-destroy"]
            plan_file, needs_application, out, err, command = build_plan(
                command,
                project_path,
                variables_args,
                state_file,
                module.params.get("targets"),
                state,
                plan_absent_args,
                plan_file,
                no_color,
            )
        diff_command = [command[0], "show", "-json", plan_file]
        rc, diff_output, err = module.run_command(diff_command, check_rc=False, cwd=project_path)
        changed, result_diff = get_diff(diff_output)
        if rc != 0:
            if workspace_ctx["current"] != workspace:
                select_workspace(command[0], project_path, workspace_ctx["current"], no_color)
            module.fail_json(
                msg=err.rstrip(),
                rc=rc,
                stdout=out,
                stdout_lines=out.splitlines(),
                stderr=err,
                stderr_lines=err.splitlines(),
                cmd=" ".join(command),
            )

    if needs_application and not module.check_mode and state != "planned":
        rc, out, err = module.run_command(command, check_rc=False, cwd=project_path)
        if rc != 0:
            if workspace_ctx["current"] != workspace:
                select_workspace(command[0], project_path, workspace_ctx["current"], no_color)
            module.fail_json(
                msg=err.rstrip(),
                rc=rc,
                stdout=out,
                stdout_lines=out.splitlines(),
                stderr=err,
                stderr_lines=err.splitlines(),
                cmd=" ".join(command),
            )
        # checks out to decide if changes were made during execution
        if " 0 added, 0 changed" not in out and state != "absent" or " 0 destroyed" not in out:
            changed = True

    if no_color:
        outputs_command = [command[0], "output", "-no-color", "-json"] + _state_args(state_file)
    else:
        outputs_command = [command[0], "output", "-json"] + _state_args(state_file)

    rc, outputs_text, outputs_err = module.run_command(outputs_command, cwd=project_path)
    outputs = {}
    if rc == 1:
        module.warn(
            f"Could not get Terraform outputs. This usually means none have been defined.\nstdout: {outputs_text}\nstderr: {outputs_err}"
        )
    elif rc != 0:
        module.fail_json(
            msg=f"Failure when getting Terraform outputs. Exited {rc}.\nstdout: {outputs_text}\nstderr: {outputs_err}",
            command=" ".join(outputs_command),
        )
    else:
        outputs = json.loads(outputs_text)

    # Restore the Terraform workspace found when running the module
    if workspace_ctx["current"] != workspace:
        select_workspace(command[0], project_path, workspace_ctx["current"], no_color)
    if state == "absent" and workspace != "default" and purge_workspace is True:
        remove_workspace(command[0], project_path, workspace, no_color)

    result = {
        "state": state,
        "workspace": workspace,
        "outputs": outputs,
        "stdout": out,
        "stderr": err,
        "command": " ".join(command),
        "changed": changed,
        "diff": result_diff,
    }

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2024, Max Maxopoly <max@dermax.org>
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: usb_facts
short_description: Allows listing information about USB devices
version_added: 8.5.0
description:
  - Allows retrieving information about available USB devices through C(lsusb).
author:
  - Max Maxopoly (@maxopoly)
extends_documentation_fragment:
  - community.general._attributes
  - community.general._attributes.facts
  - community.general._attributes.facts_module
requirements:
  - lsusb binary on PATH (usually installed through the package usbutils and preinstalled on many systems)
"""

EXAMPLES = r"""
- name: Get information about USB devices
  community.general.usb_facts:

- name: Print information about USB devices
  ansible.builtin.debug:
    msg: "On bus {{ item.bus }} device {{ item.device }} with id {{ item.id }} is {{ item.name }}"
  loop: "{{ ansible_facts.usb_devices }}"
"""

RETURN = r"""
ansible_facts:
  description: Dictionary containing details of connected USB devices.
  returned: success
  type: dict
  contains:
    usb_devices:
      description: A list of USB devices available.
      returned: success
      type: list
      elements: dict
      contains:
        bus:
          description: The bus the usb device is connected to.
          returned: success
          type: str
          sample: "001"
        device:
          description: The device number occupied on the bus.
          returned: success
          type: str
          sample: "002"
        id:
          description: ID of the USB device.
          returned: success
          type: str
          sample: "1d6b:0002"
        name:
          description: Human readable name of the device.
          returned: success
          type: str
          sample: Linux Foundation 2.0 root hub
"""

import re

from ansible.module_utils.basic import AnsibleModule


def parse_lsusb(module, lsusb_path):
    rc, stdout, stderr = module.run_command(lsusb_path, check_rc=True)
    regex = re.compile(r"^Bus (\d{3}) Device (\d{3}): ID ([0-9a-f]{4}:[0-9a-f]{4}) (.*)$")
    usb_devices = []
    for line in stdout.splitlines():
        match = re.match(regex, line)
        if not match:
            module.fail_json(msg=f"failed to parse unknown lsusb output {line}", stdout=stdout, stderr=stderr)
        current_device = {"bus": match.group(1), "device": match.group(2), "id": match.group(3), "name": match.group(4)}
        usb_devices.append(current_device)
    return_value = {"usb_devices": usb_devices}
    module.exit_json(
        msg=f"parsed {len(usb_devices)} USB devices", stdout=stdout, stderr=stderr, ansible_facts=return_value
    )


def main():
    module = AnsibleModule({}, supports_check_mode=True)

    # Set LANG env since we parse stdout
    module.run_command_environ_update = dict(LANGUAGE="C", LC_ALL="C")

    lsusb_path = module.get_bin_path("lsusb", required=True)
    parse_lsusb(module, lsusb_path)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright 2014, Max Riveiro, <kavu13@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: rollbar_deployment
author: "Max Riveiro (@kavu)"
short_description: Notify Rollbar about app deployments
description:
  - Notify Rollbar about app deployments (see U(https://rollbar.com/docs/deploys_other/)).
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  token:
    type: str
    description:
      - Your project access token.
    required: true
  environment:
    type: str
    description:
      - Name of the environment being deployed, for example V(production).
    required: true
  revision:
    type: str
    description:
      - Revision number/sha being deployed.
    required: true
  user:
    type: str
    description:
      - User who deployed.
  rollbar_user:
    type: str
    description:
      - Rollbar username of the user who deployed.
  comment:
    type: str
    description:
      - Deploy comment (for example what is being deployed).
  url:
    type: str
    description:
      - Optional URL to submit the notification to.
    default: 'https://api.rollbar.com/api/1/deploy/'
  validate_certs:
    description:
      - If V(false), SSL certificates for the target URL are not validated. This should only be used on personally controlled
        sites using self-signed certificates.
    default: true
    type: bool
"""

EXAMPLES = r"""
- name: Rollbar deployment notification
  community.general.rollbar_deployment:
  token: AAAAAA
  environment: staging
  user: ansible
  revision: '4.2'
  rollbar_user: admin
  comment: Test Deploy

- name: Notify rollbar about current git revision deployment by current user
  community.general.rollbar_deployment:
  token: "{{ rollbar_access_token }}"
  environment: production
  revision: "{{ lookup('pipe', 'git rev-parse HEAD') }}"
  user: "{{ lookup('env', 'USER') }}"
"""

import traceback
from urllib.parse import urlencode

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import fetch_url


def main():
    module = AnsibleModule(
        argument_spec=dict(
            token=dict(required=True, no_log=True),
            environment=dict(required=True),
            revision=dict(required=True),
            user=dict(),
            rollbar_user=dict(),
            comment=dict(),
            url=dict(default="https://api.rollbar.com/api/1/deploy/"),
            validate_certs=dict(default=True, type="bool"),
        ),
        supports_check_mode=True,
    )

    if module.check_mode:
        module.exit_json(changed=True)

    params = dict(
        access_token=module.params["token"],
        environment=module.params["environment"],
        revision=module.params["revision"],
    )

    if module.params["user"]:
        params["local_username"] = module.params["user"]

    if module.params["rollbar_user"]:
        params["rollbar_username"] = module.params["rollbar_user"]

    if module.params["comment"]:
        params["comment"] = module.params["comment"]

    url = module.params.get("url")

    try:
        data = urlencode(params)
        response, info = fetch_url(module, url, data=data, method="POST")
    except Exception as e:
        module.fail_json(msg=f"Unable to notify Rollbar: {e}", exception=traceback.format_exc())
    else:
        if info["status"] == 200:
            module.exit_json(changed=True)
        else:
            module.fail_json(msg=f"HTTP result code: {info['status']} connecting to {url}")


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Scaleway Serverless container registry info module
#
# Copyright (c) 2022, Guillaume MARTINEZ <lunik@tiwabbit.fr>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: scaleway_container_registry_info
short_description: Scaleway Container registry info module
version_added: 5.8.0
author: Guillaume MARTINEZ (@Lunik)
description:
  - This module return information about a container registry on Scaleway account.
extends_documentation_fragment:
  - community.general._scaleway
  - community.general._attributes
  - community.general._scaleway.actiongroup_scaleway
  - community.general._attributes.info_module

attributes:
  action_group:
    version_added: 11.3.0

options:
  project_id:
    type: str
    description:
      - Project identifier.
    required: true

  region:
    type: str
    description:
      - Scaleway region to use (for example C(fr-par)).
    required: true
    choices:
      - fr-par
      - nl-ams
      - pl-waw

  name:
    type: str
    description:
      - Name of the container registry.
    required: true
"""

EXAMPLES = r"""
- name: Get a container registry info
  community.general.scaleway_container_registry_info:
    project_id: '{{ scw_project }}'
    region: fr-par
    name: my-awesome-container-registry
  register: container_registry_info_task
"""

RETURN = r"""
container_registry:
  description: The container registry information.
  returned: always
  type: dict
  sample:
    created_at: "2022-10-14T09:51:07.949716Z"
    description: Managed by Ansible
    endpoint: rg.fr-par.scw.cloud/my-awesome-registry
    id: 0d7d5270-7864-49c2-920b-9fd6731f3589
    image_count: 0
    is_public: false
    name: my-awesome-registry
    organization_id: 10697b59-5c34-4d24-8d15-9ff2d3b89f58
    project_id: 3da4f0b2-06be-4773-8ec4-5dfa435381be
    region: fr-par
    size: 0
    status: ready
    status_message: ""
    updated_at: "2022-10-14T09:51:07.949716Z"
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._scaleway import (
    SCALEWAY_REGIONS,
    Scaleway,
    scaleway_argument_spec,
)


def info_strategy(api, wished_cn):
    cn_list = api.fetch_all_resources("namespaces")
    cn_lookup = {cn["name"]: cn for cn in cn_list}

    if wished_cn["name"] not in cn_lookup:
        msg = f"Error during container registries lookup: Unable to find container registry named '{wished_cn['name']}' in project '{wished_cn['project_id']}'"

        api.module.fail_json(msg=msg)

    target_cn = cn_lookup[wished_cn["name"]]

    response = api.get(path=f"{api.api_path}/{target_cn['id']}")
    if not response.ok:
        msg = f"Error during container registry lookup: {response.info['msg']}: '{response.json['message']}' ({response.json})"
        api.module.fail_json(msg=msg)

    return response.json


def core(module):
    region = module.params["region"]
    wished_container_namespace = {"project_id": module.params["project_id"], "name": module.params["name"]}

    api = Scaleway(module=module)
    api.api_path = f"registry/v1/regions/{region}/namespaces"

    summary = info_strategy(api=api, wished_cn=wished_container_namespace)

    module.exit_json(changed=False, container_registry=summary)


def main():
    argument_spec = scaleway_argument_spec()
    argument_spec.update(
        dict(
            project_id=dict(type="str", required=True),
            region=dict(type="str", required=True, choices=SCALEWAY_REGIONS),
            name=dict(type="str", required=True),
        )
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    core(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2015, Nate Coraor <nate@coraor.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: zfs_delegate_admin
short_description: Manage ZFS delegated administration (user admin privileges)
description:
  - Manages ZFS file system delegated administration permissions, which allow unprivileged users to perform ZFS operations
    normally restricted to the superuser.
  - See the C(zfs allow) section of V(zfs(1M\)) for detailed explanations of options.
  - This module attempts to adhere to the behavior of the command line tool as much as possible.
requirements:
  - "A ZFS/OpenZFS implementation that supports delegation with C(zfs allow), including: Solaris >= 10, illumos (all versions),
    FreeBSD >= 8.0R, ZFS on Linux >= 0.7.0."
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  name:
    description:
      - File system or volume name, for example V(rpool/myfs).
    required: true
    type: str
  state:
    description:
      - Whether to allow (V(present)), or unallow (V(absent)) a permission.
      - When set to V(present), at least one "entity" param of O(users), O(groups), or O(everyone) are required.
      - When set to V(absent), removes permissions from the specified entities, or removes all permissions if no entity params
        are specified.
    choices: [absent, present]
    default: present
    type: str
  users:
    description:
      - List of users to whom permission(s) should be granted.
    type: list
    elements: str
  groups:
    description:
      - List of groups to whom permission(s) should be granted.
    type: list
    elements: str
  everyone:
    description:
      - Apply permissions to everyone.
    type: bool
    default: false
  permissions:
    description:
      - The list of permission(s) to delegate (required if O(state=present)).
      - Supported permissions depend on the ZFS version in use. See for example U(https://openzfs.github.io/openzfs-docs/man/8/zfs-allow.8.html)
        for OpenZFS.
    type: list
    elements: str
  local:
    description:
      - Apply permissions to O(name) locally (C(zfs allow -l)).
    type: bool
  descendents:
    description:
      - Apply permissions to O(name)'s descendents (C(zfs allow -d)).
    type: bool
  recursive:
    description:
      - Unallow permissions recursively (ignored when O(state=present)).
    type: bool
    default: false
author:
  - Nate Coraor (@natefoo)
"""

EXAMPLES = r"""
- name: Grant `zfs allow` and `unallow` permission to the `adm` user with the default local+descendents scope
  community.general.zfs_delegate_admin:
    name: rpool/myfs
    users: adm
    permissions: allow,unallow

- name: Grant `zfs send` to everyone, plus the group `backup`
  community.general.zfs_delegate_admin:
    name: rpool/myvol
    groups: backup
    everyone: true
    permissions: send

- name: Grant `zfs send,receive` to users `foo` and `bar` with local scope only
  community.general.zfs_delegate_admin:
    name: rpool/myfs
    users: foo,bar
    permissions: send,receive
    local: true

- name: Revoke all permissions from everyone (permissions specifically assigned to users and groups remain)
  community.general.zfs_delegate_admin:
    name: rpool/myfs
    everyone: true
    state: absent
"""

# This module does not return anything other than the standard
# changed/state/msg/stdout
RETURN = r"""
"""

from itertools import product

from ansible.module_utils.basic import AnsibleModule


class ZfsDelegateAdmin:
    def __init__(self, module):
        self.module = module
        self.name = module.params.get("name")
        self.state = module.params.get("state")
        self.users = module.params.get("users")
        self.groups = module.params.get("groups")
        self.everyone = module.params.get("everyone")
        self.perms = module.params.get("permissions")
        self.scope = None
        self.changed = False
        self.initial_perms = None
        self.subcommand = "allow"
        self.recursive_opt = []
        self.run_method = self.update

        self.setup(module)

    def setup(self, module):
        """Validate params and set up for run."""
        if self.state == "absent":
            self.subcommand = "unallow"
            if module.params.get("recursive"):
                self.recursive_opt = ["-r"]

        local = module.params.get("local")
        descendents = module.params.get("descendents")
        if (local and descendents) or (not local and not descendents):
            self.scope = "ld"
        elif local:
            self.scope = "l"
        elif descendents:
            self.scope = "d"
        else:
            self.module.fail_json(msg="Impossible value for local and descendents")

        if not (self.users or self.groups or self.everyone):
            if self.state == "present":
                self.module.fail_json(msg="One of `users`, `groups`, or `everyone` must be set")
            elif self.state == "absent":
                self.run_method = self.clear
            # ansible ensures the else cannot happen here

        self.zfs_path = module.get_bin_path("zfs", True)

    @property
    def current_perms(self):
        """Parse the output of `zfs allow <name>` to retrieve current permissions."""
        out = self.run_zfs_raw(subcommand="allow")
        perms = {
            "l": {"u": {}, "g": {}, "e": []},
            "d": {"u": {}, "g": {}, "e": []},
            "ld": {"u": {}, "g": {}, "e": []},
        }
        linemap = {
            "Local permissions:": "l",
            "Descendent permissions:": "d",
            "Local+Descendent permissions:": "ld",
        }
        scope = None
        for line in out.splitlines():
            scope = linemap.get(line, scope)
            if not scope:
                continue
            if " (unknown: " in line:
                line = line.replace("(unknown: ", "", 1)
                line = line.replace(")", "", 1)
            try:
                if line.startswith("\tuser ") or line.startswith("\tgroup "):
                    ent_type, ent, cur_perms = line.split()
                    perms[scope][ent_type[0]][ent] = cur_perms.split(",")
                elif line.startswith("\teveryone "):
                    perms[scope]["e"] = line.split()[1].split(",")
            except ValueError:
                self.module.fail_json(msg=f"Cannot parse user/group permission output by `zfs allow`: '{line}'")
        return perms

    def run_zfs_raw(self, subcommand=None, args=None):
        """Run a raw zfs command, fail on error."""
        cmd = [self.zfs_path, subcommand or self.subcommand] + (args or []) + [self.name]
        rc, out, err = self.module.run_command(cmd)
        if rc:
            self.module.fail_json(msg=f"Command `{' '.join(cmd)}` failed: {err}")
        return out

    def run_zfs(self, args):
        """Run zfs allow/unallow with appropriate options as per module arguments."""
        args = self.recursive_opt + [f"-{self.scope}"] + args
        if self.perms:
            args.append(",".join(self.perms))
        return self.run_zfs_raw(args=args)

    def clear(self):
        """Called by run() to clear all permissions."""
        changed = False
        stdout = ""
        for scope, ent_type in product(("ld", "l", "d"), ("u", "g")):
            for ent in self.initial_perms[scope][ent_type].keys():
                stdout += self.run_zfs([f"-{ent_type}", ent])
                changed = True
        for scope in ("ld", "l", "d"):
            if self.initial_perms[scope]["e"]:
                stdout += self.run_zfs(["-e"])
                changed = True
        return (changed, stdout)

    def update(self):
        """Update permissions as per module arguments."""
        stdout = ""
        for ent_type, entities in (("u", self.users), ("g", self.groups)):
            if entities:
                stdout += self.run_zfs([f"-{ent_type}", ",".join(entities)])
        if self.everyone:
            stdout += self.run_zfs(["-e"])
        return (self.initial_perms != self.current_perms, stdout)

    def run(self):
        """Run an operation, return results for Ansible."""
        exit_args = {"state": self.state}
        self.initial_perms = self.current_perms
        exit_args["changed"], stdout = self.run_method()
        if exit_args["changed"]:
            exit_args["msg"] = "ZFS delegated admin permissions updated"
            exit_args["stdout"] = stdout
        self.module.exit_json(**exit_args)


def main():
    module = AnsibleModule(
        argument_spec=dict(
            name=dict(type="str", required=True),
            state=dict(type="str", default="present", choices=["absent", "present"]),
            users=dict(type="list", elements="str"),
            groups=dict(type="list", elements="str"),
            everyone=dict(type="bool", default=False),
            permissions=dict(type="list", elements="str"),
            local=dict(type="bool"),
            descendents=dict(type="bool"),
            recursive=dict(type="bool", default=False),
        ),
        supports_check_mode=False,
        required_if=[("state", "present", ["permissions"])],
    )
    module.run_command_environ_update = {"LANGUAGE": "C", "LC_ALL": "C"}
    zfs_delegate_admin = ZfsDelegateAdmin(module)
    zfs_delegate_admin.run()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2021, Florian Dambrine <android.florian@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: pritunl_user_info
author: "Florian Dambrine (@Lowess)"
version_added: 2.3.0
short_description: List Pritunl Users using the Pritunl API
description:
  - A module to list Pritunl users using the Pritunl API.
extends_documentation_fragment:
  - community.general._pritunl
  - community.general._attributes
  - community.general._attributes.info_module
options:
  organization:
    type: str
    required: true
    aliases:
      - org
    description:
      - The name of the organization the user is part of.
  user_name:
    type: str
    description:
      - Name of the user to filter on Pritunl.
  user_type:
    type: str
    default: client
    choices:
      - client
      - server
    description:
      - Type of the user O(user_name).
"""

EXAMPLES = r"""
- name: List all existing users part of the organization MyOrg
  community.general.pritunl_user_info:
    state: list
    organization: MyOrg

- name: Search for the user named Florian part of the organization MyOrg
  community.general.pritunl_user_info:
    state: list
    organization: MyOrg
    user_name: Florian
"""

RETURN = r"""
users:
  description: List of Pritunl users.
  returned: success
  type: list
  elements: dict
  sample:
    [
      {
        "audit": false,
        "auth_type": "google",
        "bypass_secondary": false,
        "client_to_client": false,
        "disabled": false,
        "dns_mapping": null,
        "dns_servers": null,
        "dns_suffix": null,
        "email": "foo@bar.com",
        "gravatar": true,
        "groups": [
          "foo",
          "bar"
        ],
        "id": "5d070dafe63q3b2e6s472c3b",
        "name": "foo@acme.com",
        "network_links": [],
        "organization": "58070daee6sf342e6e4s2c36",
        "organization_name": "Acme",
        "otp_auth": true,
        "otp_secret": "35H5EJA3XB2$4CWG",
        "pin": false,
        "port_forwarding": [],
        "servers": []
      }
    ]
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.dict_transformations import dict_merge

from ansible_collections.community.general.plugins.module_utils._pritunl_api import (
    PritunlException,
    get_pritunl_settings,
    list_pritunl_organizations,
    list_pritunl_users,
    pritunl_argument_spec,
)


def get_pritunl_user(module):
    user_name = module.params.get("user_name")
    user_type = module.params.get("user_type")
    org_name = module.params.get("organization")

    org_obj_list = []

    org_obj_list = list_pritunl_organizations(
        **dict_merge(get_pritunl_settings(module), {"filters": {"name": org_name}})
    )

    if len(org_obj_list) == 0:
        module.fail_json(msg=f"Can not list users from the organization '{org_name}' which does not exist")

    org_id = org_obj_list[0]["id"]

    users = list_pritunl_users(
        **dict_merge(
            get_pritunl_settings(module),
            {
                "organization_id": org_id,
                "filters": ({"type": user_type} if user_name is None else {"name": user_name, "type": user_type}),
            },
        )
    )

    result = {}
    result["changed"] = False
    result["users"] = users

    module.exit_json(**result)


def main():
    argument_spec = pritunl_argument_spec()

    argument_spec.update(
        dict(
            organization=dict(required=True, type="str", aliases=["org"]),
            user_name=dict(type="str"),
            user_type=dict(choices=["client", "server"], default="client"),
        )
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    try:
        get_pritunl_user(module)
    except PritunlException as e:
        module.fail_json(msg=f"{e}")


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2015, Benjamin Copeland (@bhcopeland) <ben@copeland.me.uk>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: statusio_maintenance
short_description: Create maintenance windows for your status.io dashboard
description:
  - Creates or deletes a maintenance window for status.io.
notes:
  - You can use the apiary API URL (U(http://docs.statusio.apiary.io/)) to capture API traffic.
  - Use start_date and start_time with minutes to set future maintenance window.
author: Benjamin Copeland (@bhcopeland) <ben@copeland.me.uk>
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  title:
    type: str
    description:
      - A descriptive title for the maintenance window.
    default: "A new maintenance window"
  desc:
    type: str
    description:
      - Message describing the maintenance window.
    default: "Created by Ansible"
  state:
    type: str
    description:
      - Desired state of the package.
    default: "present"
    choices: ["present", "absent"]
  api_id:
    type: str
    description:
      - Your unique API ID from status.io.
    required: true
  api_key:
    type: str
    description:
      - Your unique API Key from status.io.
    required: true
  statuspage:
    type: str
    description:
      - Your unique StatusPage ID from status.io.
    required: true
  url:
    type: str
    description:
      - Status.io API URL. A private apiary can be used instead.
    default: "https://api.status.io"
  components:
    type: list
    elements: str
    description:
      - The given name of your component (server name).
    aliases: ['component']
  containers:
    type: list
    elements: str
    description:
      - The given name of your container (data center).
    aliases: ['container']
  all_infrastructure_affected:
    description:
      - If it affects all components and containers.
    type: bool
    default: false
  automation:
    description:
      - Automatically start and end the maintenance window.
    type: bool
    default: false
  maintenance_notify_now:
    description:
      - Notify subscribers now.
    type: bool
    default: false
  maintenance_notify_72_hr:
    description:
      - Notify subscribers 72 hours before maintenance start time.
    type: bool
    default: false
  maintenance_notify_24_hr:
    description:
      - Notify subscribers 24 hours before maintenance start time.
    type: bool
    default: false
  maintenance_notify_1_hr:
    description:
      - Notify subscribers 1 hour before maintenance start time.
    type: bool
    default: false
  maintenance_id:
    type: str
    description:
      - The maintenance ID number when deleting a maintenance window.
  minutes:
    type: int
    description:
      - The duration of the maintenance window (starting from playbook runtime).
    default: 10
  start_date:
    type: str
    description:
      - Date maintenance is expected to start (Month/Day/Year) (UTC).
      - End Date is worked out from O(start_date) + O(minutes).
  start_time:
    type: str
    description:
      - Time maintenance is expected to start (Hour:Minutes) (UTC).
      - End Time is worked out from O(start_time) + O(minutes).
"""

EXAMPLES = r"""
- name: Create a maintenance window for 10 minutes on server1, with automation to stop the maintenance
  community.general.statusio_maintenance:
    title: Router Upgrade from ansible
    desc: Performing a Router Upgrade
    components: server1.example.com
    api_id: api_id
    api_key: api_key
    statuspage: statuspage_id
    maintenance_notify_1_hr: true
    automation: true

- name: Create a maintenance window for 60 minutes on server1 and server2
  community.general.statusio_maintenance:
    title: Routine maintenance
    desc: Some security updates
    components:
      - server1.example.com
      - server2.example.com
    minutes: 60
    api_id: api_id
    api_key: api_key
    statuspage: statuspage_id
    maintenance_notify_1_hr: true
    automation: true
  delegate_to: localhost

- name: Create a future maintenance window for 24 hours to all hosts inside the Primary Data Center
  community.general.statusio_maintenance:
    title: Data center downtime
    desc: Performing a Upgrade to our data center
    components: Primary Data Center
    api_id: api_id
    api_key: api_key
    statuspage: statuspage_id
    start_date: 01/01/2016
    start_time: 12:00
    minutes: 1440

- name: Delete a maintenance window
  community.general.statusio_maintenance:
    title: Remove a maintenance window
    maintenance_id: 561f90faf74bc94a4700087b
    statuspage: statuspage_id
    api_id: api_id
    api_key: api_key
    state: absent
"""
# TODO: Add RETURN documentation.
RETURN = """ # """

import datetime
import json

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import open_url

from ansible_collections.community.general.plugins.module_utils._datetime import (
    now,
)


def get_api_auth_headers(api_id, api_key, url, statuspage):
    headers = {"x-api-id": api_id, "x-api-key": api_key, "Content-Type": "application/json"}

    try:
        response = open_url(f"{url}/v2/component/list/{statuspage}", headers=headers)
        data = json.loads(response.read())
        if data["status"]["message"] == "Authentication failed":
            return 1, None, None, "Authentication failed: Check api_id/api_key and statuspage id."
        else:
            auth_headers = headers
            auth_content = data
    except Exception as e:
        return 1, None, None, f"{e}"
    return 0, auth_headers, auth_content, None


def get_component_ids(auth_content, components):
    host_ids = []
    lower_components = [x.lower() for x in components]
    for result in auth_content["result"]:
        if result["name"].lower() in lower_components:
            data = {"component_id": result["_id"], "container_id": result["containers"][0]["_id"]}
            host_ids.append(data)
            lower_components.remove(result["name"].lower())
    if len(lower_components):
        # items not found in the api
        return 1, None, lower_components
    return 0, host_ids, None


def get_container_ids(auth_content, containers):
    host_ids = []
    lower_containers = [x.lower() for x in containers]
    for result in auth_content["result"]:
        if result["containers"][0]["name"].lower() in lower_containers:
            data = {"component_id": result["_id"], "container_id": result["containers"][0]["_id"]}
            host_ids.append(data)
            lower_containers.remove(result["containers"][0]["name"].lower())

    if len(lower_containers):
        # items not found in the api
        return 1, None, lower_containers
    return 0, host_ids, None


def get_date_time(start_date, start_time, minutes):
    returned_date = []
    if start_date and start_time:
        try:
            datetime.datetime.strptime(start_date, "%m/%d/%Y")
            returned_date.append(start_date)
        except (NameError, ValueError):
            return 1, None, "Not a valid start_date format."
        try:
            datetime.datetime.strptime(start_time, "%H:%M")
            returned_date.append(start_time)
        except (NameError, ValueError):
            return 1, None, "Not a valid start_time format."
        try:
            # Work out end date/time based on minutes
            date_time_start = datetime.datetime.strptime(start_time + start_date, "%H:%M%m/%d/%Y")
            delta = date_time_start + datetime.timedelta(minutes=minutes)
            returned_date.append(delta.strftime("%m/%d/%Y"))
            returned_date.append(delta.strftime("%H:%M"))
        except (NameError, ValueError):
            return 1, None, "Couldn't work out a valid date"
    else:
        now_t = now()
        delta = now_t + datetime.timedelta(minutes=minutes)
        # start_date
        returned_date.append(now_t.strftime("%m/%d/%Y"))
        returned_date.append(now_t.strftime("%H:%M"))
        # end_date
        returned_date.append(delta.strftime("%m/%d/%Y"))
        returned_date.append(delta.strftime("%H:%M"))
    return 0, returned_date, None


def create_maintenance(
    auth_headers,
    url,
    statuspage,
    host_ids,
    all_infrastructure_affected,
    automation,
    title,
    desc,
    returned_date,
    maintenance_notify_now,
    maintenance_notify_72_hr,
    maintenance_notify_24_hr,
    maintenance_notify_1_hr,
):
    component_id = []
    container_id = []
    for val in host_ids:
        component_id.append(val["component_id"])
        container_id.append(val["container_id"])
    infrastructure_id = [f"{i}-{j}" for i, j in zip(component_id, container_id)]
    try:
        values = json.dumps(
            {
                "statuspage_id": statuspage,
                "all_infrastructure_affected": str(int(all_infrastructure_affected)),
                "infrastructure_affected": infrastructure_id,
                "automation": str(int(automation)),
                "maintenance_name": title,
                "maintenance_details": desc,
                "date_planned_start": returned_date[0],
                "time_planned_start": returned_date[1],
                "date_planned_end": returned_date[2],
                "time_planned_end": returned_date[3],
                "maintenance_notify_now": str(int(maintenance_notify_now)),
                "maintenance_notify_72_hr": str(int(maintenance_notify_72_hr)),
                "maintenance_notify_24_hr": str(int(maintenance_notify_24_hr)),
                "maintenance_notify_1_hr": str(int(maintenance_notify_1_hr)),
            }
        )
        response = open_url(f"{url}/v2/maintenance/schedule", data=values, headers=auth_headers)
        data = json.loads(response.read())

        if data["status"]["error"] == "yes":
            return 1, None, data["status"]["message"]
    except Exception as e:
        return 1, None, f"{e}"
    return 0, None, None


def delete_maintenance(auth_headers, url, statuspage, maintenance_id):
    try:
        values = json.dumps(
            {
                "statuspage_id": statuspage,
                "maintenance_id": maintenance_id,
            }
        )
        response = open_url(url=f"{url}/v2/maintenance/delete", data=values, headers=auth_headers)
        data = json.loads(response.read())
        if data["status"]["error"] == "yes":
            return 1, None, "Invalid maintenance_id"
    except Exception as e:
        return 1, None, f"{e}"
    return 0, None, None


def main():
    module = AnsibleModule(
        argument_spec=dict(
            api_id=dict(required=True),
            api_key=dict(required=True, no_log=True),
            statuspage=dict(required=True),
            state=dict(default="present", choices=["present", "absent"]),
            url=dict(default="https://api.status.io"),
            components=dict(type="list", elements="str", aliases=["component"]),
            containers=dict(type="list", elements="str", aliases=["container"]),
            all_infrastructure_affected=dict(type="bool", default=False),
            automation=dict(type="bool", default=False),
            title=dict(default="A new maintenance window"),
            desc=dict(default="Created by Ansible"),
            minutes=dict(type="int", default=10),
            maintenance_notify_now=dict(type="bool", default=False),
            maintenance_notify_72_hr=dict(type="bool", default=False),
            maintenance_notify_24_hr=dict(type="bool", default=False),
            maintenance_notify_1_hr=dict(type="bool", default=False),
            maintenance_id=dict(),
            start_date=dict(),
            start_time=dict(),
        ),
        supports_check_mode=True,
    )

    api_id = module.params["api_id"]
    api_key = module.params["api_key"]
    statuspage = module.params["statuspage"]
    state = module.params["state"]
    url = module.params["url"]
    components = module.params["components"]
    containers = module.params["containers"]
    all_infrastructure_affected = module.params["all_infrastructure_affected"]
    automation = module.params["automation"]
    title = module.params["title"]
    desc = module.params["desc"]
    minutes = module.params["minutes"]
    maintenance_notify_now = module.params["maintenance_notify_now"]
    maintenance_notify_72_hr = module.params["maintenance_notify_72_hr"]
    maintenance_notify_24_hr = module.params["maintenance_notify_24_hr"]
    maintenance_notify_1_hr = module.params["maintenance_notify_1_hr"]
    maintenance_id = module.params["maintenance_id"]
    start_date = module.params["start_date"]
    start_time = module.params["start_time"]

    if state == "present":
        if api_id and api_key:
            (rc, auth_headers, auth_content, error) = get_api_auth_headers(api_id, api_key, url, statuspage)
            if rc != 0:
                module.fail_json(msg=f"Failed to get auth keys: {error}")
        else:
            auth_headers = {}
            auth_content = {}

        if minutes or start_time and start_date:
            (rc, returned_date, error) = get_date_time(start_date, start_time, minutes)
            if rc != 0:
                module.fail_json(msg=f"Failed to set date/time: {error}")

        if not components and not containers:
            return module.fail_json(msg="A Component or Container must be defined")
        elif components and containers:
            return module.fail_json(msg="Components and containers cannot be used together")
        else:
            if components:
                (rc, host_ids, error) = get_component_ids(auth_content, components)
                if rc != 0:
                    module.fail_json(msg=f"Failed to find component {error}")

            if containers:
                (rc, host_ids, error) = get_container_ids(auth_content, containers)
                if rc != 0:
                    module.fail_json(msg=f"Failed to find container {error}")

        if module.check_mode:
            module.exit_json(changed=True)
        else:
            (rc, dummy, error) = create_maintenance(
                auth_headers,
                url,
                statuspage,
                host_ids,
                all_infrastructure_affected,
                automation,
                title,
                desc,
                returned_date,
                maintenance_notify_now,
                maintenance_notify_72_hr,
                maintenance_notify_24_hr,
                maintenance_notify_1_hr,
            )
            if rc == 0:
                module.exit_json(changed=True, result="Successfully created maintenance")
            else:
                module.fail_json(msg=f"Failed to create maintenance: {error}")

    if state == "absent":
        if api_id and api_key:
            (rc, auth_headers, auth_content, error) = get_api_auth_headers(api_id, api_key, url, statuspage)
            if rc != 0:
                module.fail_json(msg=f"Failed to get auth keys: {error}")
        else:
            auth_headers = {}

        if module.check_mode:
            module.exit_json(changed=True)
        else:
            (rc, dummy, error) = delete_maintenance(auth_headers, url, statuspage, maintenance_id)
            if rc == 0:
                module.exit_json(changed=True, result="Successfully deleted maintenance")
            else:
                module.fail_json(msg=f"Failed to delete maintenance: {error}")


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2017-2018 Dell EMC Inc.
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: redfish_config
short_description: Manages Out-Of-Band controllers using Redfish APIs
description:
  - Builds Redfish URIs locally and sends them to remote OOB controllers to set or update a configuration attribute.
  - Manages BIOS configuration settings.
  - Manages OOB controller configuration settings.
extends_documentation_fragment:
  - community.general._attributes
  - community.general._redfish
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  category:
    required: true
    description:
      - Category to execute on OOB controller.
    type: str
  command:
    required: true
    description:
      - List of commands to execute on OOB controller.
    type: list
    elements: str
  baseuri:
    required: true
    description:
      - Base URI of OOB controller.
    type: str
  username:
    description:
      - Username for authenticating to OOB controller.
    type: str
  password:
    description:
      - Password for authenticating to OOB controller.
    type: str
  auth_token:
    description:
      - Security token for authenticating to OOB controller.
    type: str
    version_added: 2.3.0
  bios_attributes:
    description:
      - Dictionary of BIOS attributes to update.
    default: {}
    type: dict
    version_added: '0.2.0'
  timeout:
    description:
      - Timeout in seconds for HTTP requests to OOB controller.
      - The default value for this parameter changed from V(10) to V(60) in community.general 9.0.0.
    type: int
    default: 60
  boot_order:
    description:
      - List of BootOptionReference strings specifying the BootOrder.
    default: []
    type: list
    elements: str
    version_added: '0.2.0'
  network_protocols:
    description:
      - Setting dict of manager services to update.
    type: dict
    default: {}
    version_added: '0.2.0'
  resource_id:
    description:
      - ID of the System, Manager or Chassis to modify.
    type: str
    version_added: '0.2.0'
  service_id:
    description:
      - ID of the manager to update.
    type: str
    version_added: '8.4.0'
  nic_addr:
    description:
      - EthernetInterface Address string on OOB controller.
    default: 'null'
    type: str
    version_added: '0.2.0'
  nic_config:
    description:
      - Setting dict of EthernetInterface on OOB controller.
    type: dict
    default: {}
    version_added: '0.2.0'
  strip_etag_quotes:
    description:
      - Removes surrounding quotes of etag used in C(If-Match) header of C(PATCH) requests.
      - Only use this option to resolve bad vendor implementation where C(If-Match) only matches the unquoted etag string.
    type: bool
    default: false
    version_added: 3.7.0
  hostinterface_config:
    description:
      - Setting dict of HostInterface on OOB controller.
    type: dict
    default: {}
    version_added: '4.1.0'
  hostinterface_id:
    description:
      - Redfish HostInterface instance ID if multiple HostInterfaces are present.
    type: str
    version_added: '4.1.0'
  sessions_config:
    description:
      - Setting dict of Sessions.
    type: dict
    default: {}
    version_added: '5.7.0'
  storage_subsystem_id:
    description:
      - ID of the Storage Subsystem on which the volume is to be created.
    type: str
    default: ''
    version_added: '7.3.0'
  storage_none_volume_deletion:
    description:
      - Indicates if all non-RAID volumes are automatically deleted prior to creating the new volume.
    type: bool
    default: false
    version_added: '9.5.0'
  volume_ids:
    description:
      - List of IDs of volumes to be deleted.
    type: list
    default: []
    elements: str
    version_added: '7.3.0'
  secure_boot_enable:
    description:
      - Setting parameter to enable or disable SecureBoot.
    type: bool
    default: true
    version_added: '7.5.0'
  volume_details:
    description:
      - Setting dictionary of volume to be created.
      - If C(CapacityBytes) key is not specified in this dictionary, the size of the volume is determined by the Redfish service.
        It is possible the size is not the maximum available size.
    type: dict
    default: {}
    version_added: '7.5.0'
  power_restore_policy:
    description:
      - The desired power state of the system when power is restored after a power loss.
    type: str
    choices:
      - AlwaysOn
      - AlwaysOff
      - LastState
    version_added: '10.5.0'
  ciphers:
    version_added: 9.2.0
  validate_certs:
    version_added: 10.6.0
  ca_path:
    version_added: 10.6.0

author:
  - "Jose Delarosa (@jose-delarosa)"
  - "T S Kushal (@TSKushal)"
"""

EXAMPLES = r"""
- name: Set BootMode to UEFI
  community.general.redfish_config:
    category: Systems
    command: SetBiosAttributes
    resource_id: 437XR1138R2
    bios_attributes:
      BootMode: "Uefi"
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Set multiple BootMode attributes
  community.general.redfish_config:
    category: Systems
    command: SetBiosAttributes
    resource_id: 437XR1138R2
    bios_attributes:
      BootMode: "Bios"
      OneTimeBootMode: "Enabled"
      BootSeqRetry: "Enabled"
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Enable PXE Boot for NIC1
  community.general.redfish_config:
    category: Systems
    command: SetBiosAttributes
    resource_id: 437XR1138R2
    bios_attributes:
      PxeDev1EnDis: Enabled
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Set BIOS default settings with a timeout of 20 seconds
  community.general.redfish_config:
    category: Systems
    command: SetBiosDefaultSettings
    resource_id: 437XR1138R2
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"
    timeout: 20

- name: Set boot order
  community.general.redfish_config:
    category: Systems
    command: SetBootOrder
    boot_order:
      - Boot0002
      - Boot0001
      - Boot0000
      - Boot0003
      - Boot0004
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Set boot order to the default
  community.general.redfish_config:
    category: Systems
    command: SetDefaultBootOrder
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Set Manager Network Protocols
  community.general.redfish_config:
    category: Manager
    command: SetNetworkProtocols
    network_protocols:
      SNMP:
        ProtocolEnabled: true
        Port: 161
      HTTP:
        ProtocolEnabled: false
        Port: 8080
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Set Manager NIC
  community.general.redfish_config:
    category: Manager
    command: SetManagerNic
    nic_config:
      DHCPv4:
        DHCPEnabled: false
      IPv4StaticAddresses:
        Address: 192.168.1.3
        Gateway: 192.168.1.1
        SubnetMask: 255.255.255.0
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Disable Host Interface
  community.general.redfish_config:
    category: Manager
    command: SetHostInterface
    hostinterface_config:
      InterfaceEnabled: false
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Enable Host Interface for HostInterface resource ID '2'
  community.general.redfish_config:
    category: Manager
    command: SetHostInterface
    hostinterface_config:
      InterfaceEnabled: true
    hostinterface_id: "2"
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Set SessionService Session Timeout to 30 minutes
  community.general.redfish_config:
    category: Sessions
    command: SetSessionService
    sessions_config:
      SessionTimeout: 1800
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Enable SecureBoot
  community.general.redfish_config:
    category: Systems
    command: EnableSecureBoot
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"

- name: Set SecureBoot
  community.general.redfish_config:
    category: Systems
    command: SetSecureBoot
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"
    secure_boot_enable: true

- name: Delete All Volumes
  community.general.redfish_config:
    category: Systems
    command: DeleteVolumes
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"
    storage_subsystem_id: "DExxxxxx"
    volume_ids: ["volume1", "volume2"]

- name: Create Volume
  community.general.redfish_config:
    category: Systems
    command: CreateVolume
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"
    storage_subsystem_id: "DExxxxxx"
    volume_details:
      Name: "MR Volume"
      RAIDType: "RAID0"
      Drives:
        - "/redfish/v1/Systems/1/Storage/DE00B000/Drives/1"

- name: Set PowerRestorePolicy
  community.general.redfish_config:
    category: Systems
    command: SetPowerRestorePolicy
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"
    power_restore_policy: "AlwaysOff"

- name: Set service identification to {{ service_id }}
  community.general.redfish_config:
    category: Manager
    command: SetServiceIdentification
    service_id: "{{ service_id }}"
    baseuri: "{{ baseuri }}"
    username: "{{ username }}"
    password: "{{ password }}"
"""

RETURN = r"""
msg:
  description: Message with action result or error description.
  returned: always
  type: str
  sample: "Action was successful"
"""

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_native

from ansible_collections.community.general.plugins.module_utils._redfish_utils import (
    REDFISH_COMMON_ARGUMENT_SPEC,
    RedfishUtils,
)

# More will be added as module features are expanded
CATEGORY_COMMANDS_ALL = {
    "Systems": [
        "SetBiosDefaultSettings",
        "SetBiosAttributes",
        "SetBootOrder",
        "SetDefaultBootOrder",
        "EnableSecureBoot",
        "SetSecureBoot",
        "DeleteVolumes",
        "CreateVolume",
        "SetPowerRestorePolicy",
    ],
    "Manager": ["SetNetworkProtocols", "SetManagerNic", "SetHostInterface", "SetServiceIdentification"],
    "Sessions": ["SetSessionService"],
}


def main():
    result = {}
    argument_spec = dict(
        category=dict(required=True),
        command=dict(required=True, type="list", elements="str"),
        baseuri=dict(required=True),
        username=dict(),
        password=dict(no_log=True),
        auth_token=dict(no_log=True),
        bios_attributes=dict(type="dict", default={}),
        timeout=dict(type="int", default=60),
        boot_order=dict(type="list", elements="str", default=[]),
        network_protocols=dict(type="dict", default={}),
        resource_id=dict(),
        service_id=dict(),
        nic_addr=dict(default="null"),
        nic_config=dict(type="dict", default={}),
        strip_etag_quotes=dict(type="bool", default=False),
        hostinterface_config=dict(type="dict", default={}),
        hostinterface_id=dict(),
        sessions_config=dict(type="dict", default={}),
        storage_subsystem_id=dict(type="str", default=""),
        storage_none_volume_deletion=dict(type="bool", default=False),
        volume_ids=dict(type="list", default=[], elements="str"),
        secure_boot_enable=dict(type="bool", default=True),
        volume_details=dict(type="dict", default={}),
        power_restore_policy=dict(choices=["AlwaysOn", "AlwaysOff", "LastState"]),
    )
    argument_spec.update(REDFISH_COMMON_ARGUMENT_SPEC)
    module = AnsibleModule(
        argument_spec,
        required_together=[
            ("username", "password"),
        ],
        required_one_of=[
            ("username", "auth_token"),
        ],
        mutually_exclusive=[
            ("username", "auth_token"),
        ],
        supports_check_mode=False,
    )

    category = module.params["category"]
    command_list = module.params["command"]

    # admin credentials used for authentication
    creds = {"user": module.params["username"], "pswd": module.params["password"], "token": module.params["auth_token"]}

    # timeout
    timeout = module.params["timeout"]

    # BIOS attributes to update
    bios_attributes = module.params["bios_attributes"]

    # boot order
    boot_order = module.params["boot_order"]

    # System, Manager or Chassis ID to modify
    resource_id = module.params["resource_id"]

    # manager nic
    nic_addr = module.params["nic_addr"]
    nic_config = module.params["nic_config"]

    # Etag options
    strip_etag_quotes = module.params["strip_etag_quotes"]

    # HostInterface config options
    hostinterface_config = module.params["hostinterface_config"]

    # HostInterface instance ID
    hostinterface_id = module.params["hostinterface_id"]

    # Service Identification
    service_id = module.params["service_id"]

    # Sessions config options
    sessions_config = module.params["sessions_config"]

    # Volume deletion options
    storage_subsystem_id = module.params["storage_subsystem_id"]
    volume_ids = module.params["volume_ids"]

    # Set SecureBoot options
    secure_boot_enable = module.params["secure_boot_enable"]

    # Volume creation options
    volume_details = module.params["volume_details"]
    storage_subsystem_id = module.params["storage_subsystem_id"]
    storage_none_volume_deletion = module.params["storage_none_volume_deletion"]

    # Power Restore Policy
    power_restore_policy = module.params["power_restore_policy"]

    # Build root URI
    root_uri = f"https://{module.params['baseuri']}"
    rf_utils = RedfishUtils(
        creds,
        root_uri,
        timeout,
        module,
        resource_id=resource_id,
        data_modification=True,
        strip_etag_quotes=strip_etag_quotes,
    )

    # Check that Category is valid
    if category not in CATEGORY_COMMANDS_ALL:
        module.fail_json(msg=f"Invalid Category '{category}'. Valid Categories = {list(CATEGORY_COMMANDS_ALL.keys())}")

    # Check that all commands are valid
    for cmd in command_list:
        # Fail if even one command given is invalid
        if cmd not in CATEGORY_COMMANDS_ALL[category]:
            module.fail_json(msg=f"Invalid Command '{cmd}'. Valid Commands = {CATEGORY_COMMANDS_ALL[category]}")

    # Organize by Categories / Commands
    if category == "Systems":
        # execute only if we find a System resource
        result = rf_utils._find_systems_resource()
        if result["ret"] is False:
            module.fail_json(msg=result["msg"])

        for command in command_list:
            if command == "SetBiosDefaultSettings":
                result = rf_utils.set_bios_default_settings()
            elif command == "SetBiosAttributes":
                result = rf_utils.set_bios_attributes(bios_attributes)
            elif command == "SetBootOrder":
                result = rf_utils.set_boot_order(boot_order)
            elif command == "SetDefaultBootOrder":
                result = rf_utils.set_default_boot_order()
            elif command == "EnableSecureBoot":
                result = rf_utils.enable_secure_boot()
            elif command == "SetSecureBoot":
                result = rf_utils.set_secure_boot(secure_boot_enable)
            elif command == "DeleteVolumes":
                result = rf_utils.delete_volumes(storage_subsystem_id, volume_ids)
            elif command == "CreateVolume":
                result = rf_utils.create_volume(volume_details, storage_subsystem_id, storage_none_volume_deletion)
            elif command == "SetPowerRestorePolicy":
                result = rf_utils.set_power_restore_policy(power_restore_policy)

    elif category == "Manager":
        # execute only if we find a Manager service resource
        result = rf_utils._find_managers_resource()
        if result["ret"] is False:
            module.fail_json(msg=result["msg"])

        for command in command_list:
            if command == "SetNetworkProtocols":
                result = rf_utils.set_network_protocols(module.params["network_protocols"])
            elif command == "SetManagerNic":
                result = rf_utils.set_manager_nic(nic_addr, nic_config)
            elif command == "SetHostInterface":
                result = rf_utils.set_hostinterface_attributes(hostinterface_config, hostinterface_id)
            elif command == "SetServiceIdentification":
                result = rf_utils.set_service_identification(service_id)

    elif category == "Sessions":
        # execute only if we find a Sessions resource
        result = rf_utils._find_sessionservice_resource()
        if result["ret"] is False:
            module.fail_json(msg=result["msg"])

        for command in command_list:
            if command == "SetSessionService":
                result = rf_utils.set_session_service(sessions_config)

    # Return data back or fail with proper message
    if result["ret"] is True:
        if result.get("warning"):
            module.warn(to_native(result["warning"]))

        module.exit_json(changed=result["changed"], msg=to_native(result["msg"]))
    else:
        module.fail_json(msg=to_native(result["msg"]))


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Scaleway Security Group Rule management module
#
# Copyright (C) 2018 Antoine Barbare (antoinebarbare@gmail.com).
#
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: scaleway_security_group_rule
short_description: Scaleway Security Group Rule management module
author: Antoine Barbare (@abarbare)
description:
  - This module manages Security Group Rule on Scaleway account U(https://developer.scaleway.com).
extends_documentation_fragment:
  - community.general._scaleway
  - community.general._attributes
  - community.general._scaleway.actiongroup_scaleway

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
  action_group:
    version_added: 11.3.0

options:
  state:
    type: str
    description:
      - Indicate desired state of the Security Group Rule.
    default: present
    choices:
      - present
      - absent

  region:
    type: str
    description:
      - Scaleway region to use (for example V(par1)).
    required: true
    choices:
      - ams1
      - EMEA-NL-EVS
      - ams2
      - ams3
      - par1
      - EMEA-FR-PAR1
      - par2
      - EMEA-FR-PAR2
      - par3
      - waw1
      - EMEA-PL-WAW1
      - waw2
      - waw3

  protocol:
    type: str
    description:
      - Network protocol to use.
    choices:
      - TCP
      - UDP
      - ICMP
    required: true

  port:
    description:
      - Port related to the rule, null value for all the ports.
    required: true
    type: int

  ip_range:
    type: str
    description:
      - IPV4 CIDR notation to apply to the rule.
    default: 0.0.0.0/0

  direction:
    type: str
    description:
      - Rule direction.
    choices:
      - inbound
      - outbound
    required: true

  action:
    type: str
    description:
      - Rule action.
    choices:
      - accept
      - drop
    required: true

  security_group:
    type: str
    description:
      - Security Group unique identifier.
    required: true
"""

EXAMPLES = r"""
- name: Create a Security Group Rule
  community.general.scaleway_security_group_rule:
    state: present
    region: par1
    protocol: TCP
    port: 80
    ip_range: 0.0.0.0/0
    direction: inbound
    action: accept
    security_group: b57210ee-1281-4820-a6db-329f78596ecb
  register: security_group_rule_creation_task
"""

RETURN = r"""
data:
  description: This is only present when O(state=present).
  returned: when O(state=present)
  type: dict
  sample:
    {
      "scaleway_security_group_rule": {
        "direction": "inbound",
        "protocol": "TCP",
        "ip_range": "0.0.0.0/0",
        "dest_port_from": 80,
        "action": "accept",
        "position": 2,
        "dest_port_to": null,
        "editable": null,
        "id": "10cb0b9a-80f6-4830-abd7-a31cd828b5e9"
      }
    }
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._scaleway import (
    SCALEWAY_LOCATION,
    Scaleway,
    payload_from_object,
    scaleway_argument_spec,
)


def get_sgr_from_api(security_group_rules, security_group_rule):
    """Check if a security_group_rule specs are present in security_group_rules
    Return None if no rules match the specs
    Return the rule if found
    """
    for sgr in security_group_rules:
        if (
            sgr["ip_range"] == security_group_rule["ip_range"]
            and sgr["dest_port_from"] == security_group_rule["dest_port_from"]
            and sgr["direction"] == security_group_rule["direction"]
            and sgr["action"] == security_group_rule["action"]
            and sgr["protocol"] == security_group_rule["protocol"]
        ):
            return sgr

    return None


def present_strategy(api, security_group_id, security_group_rule):
    ret = {"changed": False}

    response = api.get(f"security_groups/{security_group_id}/rules")
    if not response.ok:
        api.module.fail_json(
            msg=f'Error getting security group rules "{response.info["msg"]}": "{response.json["message"]}" ({response.json})'
        )

    existing_rule = get_sgr_from_api(response.json["rules"], security_group_rule)

    if not existing_rule:
        ret["changed"] = True
        if api.module.check_mode:
            return ret

        # Create Security Group Rule
        response = api.post(
            f"/security_groups/{security_group_id}/rules", data=payload_from_object(security_group_rule)
        )

        if not response.ok:
            api.module.fail_json(
                msg=f'Error during security group rule creation: "{response.info["msg"]}": "{response.json["message"]}" ({response.json})'
            )
        ret["scaleway_security_group_rule"] = response.json["rule"]

    else:
        ret["scaleway_security_group_rule"] = existing_rule

    return ret


def absent_strategy(api, security_group_id, security_group_rule):
    ret = {"changed": False}

    response = api.get(f"security_groups/{security_group_id}/rules")
    if not response.ok:
        api.module.fail_json(
            msg=f'Error getting security group rules "{response.info["msg"]}": "{response.json["message"]}" ({response.json})'
        )

    existing_rule = get_sgr_from_api(response.json["rules"], security_group_rule)

    if not existing_rule:
        return ret

    ret["changed"] = True
    if api.module.check_mode:
        return ret

    response = api.delete(f"/security_groups/{security_group_id}/rules/{existing_rule['id']}")
    if not response.ok:
        api.module.fail_json(
            msg=f'Error deleting security group rule "{response.info["msg"]}": "{response.json["message"]}" ({response.json})'
        )

    return ret


def core(module):
    api = Scaleway(module=module)

    security_group_rule = {
        "protocol": module.params["protocol"],
        "dest_port_from": module.params["port"],
        "ip_range": module.params["ip_range"],
        "direction": module.params["direction"],
        "action": module.params["action"],
    }

    region = module.params["region"]
    module.params["api_url"] = SCALEWAY_LOCATION[region]["api_endpoint"]

    if module.params["state"] == "present":
        summary = present_strategy(
            api=api, security_group_id=module.params["security_group"], security_group_rule=security_group_rule
        )
    else:
        summary = absent_strategy(
            api=api, security_group_id=module.params["security_group"], security_group_rule=security_group_rule
        )
    module.exit_json(**summary)


def main():
    argument_spec = scaleway_argument_spec()
    argument_spec.update(
        state=dict(type="str", default="present", choices=["absent", "present"]),
        region=dict(type="str", required=True, choices=list(SCALEWAY_LOCATION.keys())),
        protocol=dict(type="str", required=True, choices=["TCP", "UDP", "ICMP"]),
        port=dict(type="int", required=True),
        ip_range=dict(type="str", default="0.0.0.0/0"),
        direction=dict(type="str", required=True, choices=["inbound", "outbound"]),
        action=dict(type="str", required=True, choices=["accept", "drop"]),
        security_group=dict(type="str", required=True),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    core(module)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, Milan Ilic <milani@nordeus.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Make coding more python3-ish
from __future__ import annotations

DOCUMENTATION = r"""
module: one_service
short_description: Deploy and manage OpenNebula services
description:
  - Manage OpenNebula services.
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  api_url:
    description:
      - URL of the OpenNebula OneFlow API server.
      - It is recommended to use HTTPS so that the username/password are not transferred over the network unencrypted.
      - If not set then the value of the E(ONEFLOW_URL) environment variable is used.
    type: str
  api_username:
    description:
      - Name of the user to login into the OpenNebula OneFlow API server. If not set then the value of the E(ONEFLOW_USERNAME)
        environment variable is used.
    type: str
  api_password:
    description:
      - Password of the user to login into OpenNebula OneFlow API server. If not set then the value of the E(ONEFLOW_PASSWORD)
        environment variable is used.
    type: str
  template_name:
    description:
      - Name of service template to use to create a new instance of a service.
    type: str
  template_id:
    description:
      - ID of a service template to use to create a new instance of a service.
    type: int
  service_id:
    description:
      - ID of a service instance that you would like to manage.
    type: int
  service_name:
    description:
      - Name of a service instance that you would like to manage.
    type: str
  unique:
    description:
      - Setting O(unique=true) ensures that there is only one service instance running with a name set with O(service_name)
        when instantiating a service from a template specified with O(template_id) or O(template_name). Check examples below.
    type: bool
    default: false
  state:
    description:
      - V(present) - instantiate a service from a template specified with O(template_id) or O(template_name).
      - V(absent) - terminate an instance of a service specified with O(template_id) or O(template_name).
    choices: ["present", "absent"]
    default: present
    type: str
  mode:
    description:
      - Set permission mode of a service instance in octet format, for example V(0600) to give owner C(use) and C(manage)
        and nothing to group and others.
    type: str
  owner_id:
    description:
      - ID of the user which is set as the owner of the service.
    type: int
  group_id:
    description:
      - ID of the group which is set as the group of the service.
    type: int
  wait:
    description:
      - Wait for the instance to reach RUNNING state after DEPLOYING or COOLDOWN state after SCALING.
    type: bool
    default: false
  wait_timeout:
    description:
      - How long before wait gives up, in seconds.
    default: 300
    type: int
  custom_attrs:
    description:
      - Dictionary of key/value custom attributes which is used when instantiating a new service.
    default: {}
    type: dict
  role:
    description:
      - Name of the role whose cardinality should be changed.
    type: str
  cardinality:
    description:
      - Number of VMs for the specified role.
    type: int
  force:
    description:
      - Force the new cardinality even if it is outside the limits.
    type: bool
    default: false
author:
  - "Milan Ilic (@ilicmilan)"
"""

EXAMPLES = r"""
- name: Instantiate a new service
  community.general.one_service:
    template_id: 90
  register: result

- name: Print service properties
  ansible.builtin.debug:
    msg: result

- name: Instantiate a new service with specified service_name, service group and mode
  community.general.one_service:
    template_name: 'app1_template'
    service_name: 'app1'
    group_id: 1
    mode: '660'

- name: Instantiate a new service with template_id and pass custom_attrs dict
  community.general.one_service:
    template_id: 90
    custom_attrs:
      public_network_id: 21
      private_network_id: 26

- name: Instantiate a new service 'foo' if the service doesn't already exist, otherwise do nothing
  community.general.one_service:
    template_id: 53
    service_name: 'foo'
    unique: true

- name: Delete a service by ID
  community.general.one_service:
    service_id: 153
    state: absent

- name: Get service info
  community.general.one_service:
    service_id: 153
  register: service_info

- name: Change service owner, group and mode
  community.general.one_service:
    service_name: 'app2'
    owner_id: 34
    group_id: 113
    mode: '600'

- name: Instantiate service and wait for it to become RUNNING
  community.general.one_service:
    template_id: 43
    service_name: 'foo1'

- name: Wait service to become RUNNING
  community.general.one_service:
    service_id: 112
    wait: true

- name: Change role cardinality
  community.general.one_service:
    service_id: 153
    role: bar
    cardinality: 5

- name: Change role cardinality and wait for it to be applied
  community.general.one_service:
    service_id: 112
    role: foo
    cardinality: 7
    wait: true
"""

RETURN = r"""
service_id:
  description: Service ID.
  type: int
  returned: success
  sample: 153
service_name:
  description: Service name.
  type: str
  returned: success
  sample: app1
group_id:
  description: Service's group ID.
  type: int
  returned: success
  sample: 1
group_name:
  description: Service's group name.
  type: str
  returned: success
  sample: one-users
owner_id:
  description: Service's owner ID.
  type: int
  returned: success
  sample: 143
owner_name:
  description: Service's owner name.
  type: str
  returned: success
  sample: ansible-test
state:
  description: State of service instance.
  type: str
  returned: success
  sample: RUNNING
mode:
  description: Service's mode.
  type: int
  returned: success
  sample: 660
roles:
  description: List of dictionaries of roles, each role is described by name, cardinality, state and nodes IDs.
  type: list
  returned: success
  sample:
    - {"cardinality": 1, "name": "foo", "state": "RUNNING", "ids": [123, 456]}
    - {"cardinality": 2, "name": "bar", "state": "RUNNING", "ids": [452, 567, 746]}
"""

import os
import time
from dataclasses import dataclass

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.urls import open_url


@dataclass
class AuthParams:
    url: str
    user: str
    password: str


STATES = (
    "PENDING",
    "DEPLOYING",
    "RUNNING",
    "UNDEPLOYING",
    "WARNING",
    "DONE",
    "FAILED_UNDEPLOYING",
    "FAILED_DEPLOYING",
    "SCALING",
    "FAILED_SCALING",
    "COOLDOWN",
)


def get_all_templates(module, auth):
    try:
        all_templates = open_url(
            url=f"{auth.url}/service_template",
            method="GET",
            force_basic_auth=True,
            url_username=auth.user,
            url_password=auth.password,
        )
    except Exception as e:
        module.fail_json(msg=str(e))

    return module.from_json(all_templates.read())


def get_template(module, auth, pred):
    all_templates_dict = get_all_templates(module, auth)

    found = 0
    found_template = None
    template_name = ""

    if "DOCUMENT_POOL" in all_templates_dict and "DOCUMENT" in all_templates_dict["DOCUMENT_POOL"]:
        for template in all_templates_dict["DOCUMENT_POOL"]["DOCUMENT"]:
            if pred(template):
                found = found + 1
                found_template = template
                template_name = template["NAME"]

    if found <= 0:
        return None
    elif found > 1:
        module.fail_json(msg=f"There is no template with unique name: {template_name}")
    else:
        return found_template


def get_all_services(module, auth):
    try:
        response = open_url(
            f"{auth.url}/service",
            method="GET",
            force_basic_auth=True,
            url_username=auth.user,
            url_password=auth.password,
        )
    except Exception as e:
        module.fail_json(msg=str(e))

    return module.from_json(response.read())


def get_service(module, auth, pred):
    all_services_dict = get_all_services(module, auth)

    found = 0
    found_service = None
    service_name = ""

    if "DOCUMENT_POOL" in all_services_dict and "DOCUMENT" in all_services_dict["DOCUMENT_POOL"]:
        for service in all_services_dict["DOCUMENT_POOL"]["DOCUMENT"]:
            if pred(service):
                found = found + 1
                found_service = service
                service_name = service["NAME"]

    # fail if there are more services with same name
    if found > 1:
        module.fail_json(
            msg=f"There are multiple services with a name: '{service_name}'. You have to use a unique service name or use 'service_id' instead."
        )
    elif found <= 0:
        return None
    else:
        return found_service


def get_service_by_id(module, auth, service_id):
    return get_service(module, auth, lambda service: int(service["ID"]) == int(service_id)) if service_id else None


def get_service_by_name(module, auth, service_name):
    return get_service(module, auth, lambda service: service["NAME"] == service_name)


def get_service_info(module, auth, service):
    result = {
        "service_id": int(service["ID"]),
        "service_name": service["NAME"],
        "group_id": int(service["GID"]),
        "group_name": service["GNAME"],
        "owner_id": int(service["UID"]),
        "owner_name": service["UNAME"],
        "state": STATES[service["TEMPLATE"]["BODY"]["state"]],
    }

    roles_status = service["TEMPLATE"]["BODY"]["roles"]
    roles = []
    for role in roles_status:
        nodes_ids = []
        if "nodes" in role:
            for node in role["nodes"]:
                nodes_ids.append(node["deploy_id"])
        roles.append(
            {
                "name": role["name"],
                "cardinality": role["cardinality"],
                "state": STATES[int(role["state"])],
                "ids": nodes_ids,
            }
        )

    result["roles"] = roles
    result["mode"] = int(parse_service_permissions(service))

    return result


def create_service(module, auth, template_id, service_name, custom_attrs, unique, wait, wait_timeout):
    # make sure that the values in custom_attrs dict are strings
    custom_attrs_with_str = {k: str(v) for k, v in custom_attrs.items()}

    data = {
        "action": {
            "perform": "instantiate",
            "params": {"merge_template": {"custom_attrs_values": custom_attrs_with_str, "name": service_name}},
        }
    }

    try:
        response = open_url(
            f"{auth.url}/service_template/{template_id!s}/action",
            method="POST",
            data=module.jsonify(data),
            force_basic_auth=True,
            url_username=auth.user,
            url_password=auth.password,
        )
    except Exception as e:
        module.fail_json(msg=str(e))

    service_result = module.from_json(response.read())["DOCUMENT"]

    return service_result


def wait_for_service_to_become_ready(module, auth, service_id, wait_timeout):
    start_time = time.time()

    while (time.time() - start_time) < wait_timeout:
        try:
            status_result = open_url(
                f"{auth.url}/service/{service_id!s}",
                method="GET",
                force_basic_auth=True,
                url_username=auth.user,
                url_password=auth.password,
            )
        except Exception as e:
            module.fail_json(msg=f"Request for service status has failed. Error message: {e!s}")

        status_result = module.from_json(status_result.read())
        service_state = status_result["DOCUMENT"]["TEMPLATE"]["BODY"]["state"]

        if service_state in [STATES.index("RUNNING"), STATES.index("COOLDOWN")]:
            return status_result["DOCUMENT"]
        elif service_state not in [STATES.index("PENDING"), STATES.index("DEPLOYING"), STATES.index("SCALING")]:
            log_message = ""
            for log_info in status_result["DOCUMENT"]["TEMPLATE"]["BODY"]["log"]:
                if log_info["severity"] == "E":
                    log_message = log_message + log_info["message"]
                    break

            module.fail_json(
                msg=f"Deploying is unsuccessful. Service state: {STATES[service_state]}. Error message: {log_message}"
            )

        time.sleep(1)

    module.fail_json(msg="Wait timeout has expired")


def change_service_permissions(module, auth, service_id, permissions):
    data = {"action": {"perform": "chmod", "params": {"octet": permissions}}}

    try:
        open_url(
            f"{auth.url}/service/{service_id!s}/action",
            method="POST",
            force_basic_auth=True,
            url_username=auth.user,
            url_password=auth.password,
            data=module.jsonify(data),
        )
    except Exception as e:
        module.fail_json(msg=str(e))


def change_service_owner(module, auth, service_id, owner_id):
    data = {"action": {"perform": "chown", "params": {"owner_id": owner_id}}}

    try:
        open_url(
            f"{auth.url}/service/{service_id!s}/action",
            method="POST",
            force_basic_auth=True,
            url_username=auth.user,
            url_password=auth.password,
            data=module.jsonify(data),
        )
    except Exception as e:
        module.fail_json(msg=str(e))


def change_service_group(module, auth, service_id, group_id):
    data = {"action": {"perform": "chgrp", "params": {"group_id": group_id}}}

    try:
        open_url(
            f"{auth.url}/service/{service_id!s}/action",
            method="POST",
            force_basic_auth=True,
            url_username=auth.user,
            url_password=auth.password,
            data=module.jsonify(data),
        )
    except Exception as e:
        module.fail_json(msg=str(e))


def change_role_cardinality(module, auth, service_id, role, cardinality, force):
    data = {"cardinality": cardinality, "force": force}

    try:
        status_result = open_url(
            f"{auth.url}/service/{service_id!s}/role/{role}",
            method="PUT",
            force_basic_auth=True,
            url_username=auth.user,
            url_password=auth.password,
            data=module.jsonify(data),
        )
    except Exception as e:
        module.fail_json(msg=str(e))

    if status_result.getcode() != 204:
        module.fail_json(msg=f"Failed to change cardinality for role: {role}. Return code: {status_result.getcode()!s}")


def check_change_service_owner(module, service, owner_id):
    old_owner_id = int(service["UID"])

    return old_owner_id != owner_id


def check_change_service_group(module, service, group_id):
    old_group_id = int(service["GID"])

    return old_group_id != group_id


def parse_service_permissions(service):
    perm_dict = service["PERMISSIONS"]
    """
    This is the structure of the 'PERMISSIONS' dictionary:

   "PERMISSIONS": {
                      "OWNER_U": "1",
                      "OWNER_M": "1",
                      "OWNER_A": "0",
                      "GROUP_U": "0",
                      "GROUP_M": "0",
                      "GROUP_A": "0",
                      "OTHER_U": "0",
                      "OTHER_M": "0",
                      "OTHER_A": "0"
                    }
    """

    owner_octal = int(perm_dict["OWNER_U"]) * 4 + int(perm_dict["OWNER_M"]) * 2 + int(perm_dict["OWNER_A"])
    group_octal = int(perm_dict["GROUP_U"]) * 4 + int(perm_dict["GROUP_M"]) * 2 + int(perm_dict["GROUP_A"])
    other_octal = int(perm_dict["OTHER_U"]) * 4 + int(perm_dict["OTHER_M"]) * 2 + int(perm_dict["OTHER_A"])

    permissions = str(owner_octal) + str(group_octal) + str(other_octal)

    return permissions


def check_change_service_permissions(module, service, permissions):
    old_permissions = parse_service_permissions(service)

    return old_permissions != permissions


def check_change_role_cardinality(module, service, role_name, cardinality):
    roles_list = service["TEMPLATE"]["BODY"]["roles"]

    for role in roles_list:
        if role["name"] == role_name:
            return int(role["cardinality"]) != cardinality

    module.fail_json(msg=f"There is no role with name: {role_name}")


def create_service_and_operation(
    module, auth, template_id, service_name, owner_id, group_id, permissions, custom_attrs, unique, wait, wait_timeout
):
    if not service_name:
        service_name = ""
    changed = False
    service = None

    if unique:
        service = get_service_by_name(module, auth, service_name)

    if not service or service["TEMPLATE"]["BODY"]["state"] == "DONE":
        if not module.check_mode:
            service = create_service(module, auth, template_id, service_name, custom_attrs, unique, wait, wait_timeout)
        changed = True

    # if check_mode=true and there would be changes, service doesn't exist and we can not get it
    if module.check_mode and changed:
        return {"changed": True}

    result = service_operation(
        module,
        auth,
        owner_id=owner_id,
        group_id=group_id,
        wait=wait,
        wait_timeout=wait_timeout,
        permissions=permissions,
        service=service,
    )

    if result["changed"]:
        changed = True

    result["changed"] = changed

    return result


def service_operation(
    module,
    auth,
    service_id=None,
    owner_id=None,
    group_id=None,
    permissions=None,
    role=None,
    cardinality=None,
    force=None,
    wait=False,
    wait_timeout=None,
    service=None,
):
    changed = False

    if not service:
        service = get_service_by_id(module, auth, service_id)
    else:
        service_id = service["ID"]

    if not service:
        module.fail_json(msg=f"There is no service with id: {service_id!s}")

    if owner_id:
        if check_change_service_owner(module, service, owner_id):
            if not module.check_mode:
                change_service_owner(module, auth, service_id, owner_id)
            changed = True
    if group_id:
        if check_change_service_group(module, service, group_id):
            if not module.check_mode:
                change_service_group(module, auth, service_id, group_id)
            changed = True
    if permissions:
        if check_change_service_permissions(module, service, permissions):
            if not module.check_mode:
                change_service_permissions(module, auth, service_id, permissions)
            changed = True

    if role:
        if check_change_role_cardinality(module, service, role, cardinality):
            if not module.check_mode:
                change_role_cardinality(module, auth, service_id, role, cardinality, force)
            changed = True

    if wait and not module.check_mode:
        service = wait_for_service_to_become_ready(module, auth, service_id, wait_timeout)

    # if something has changed, fetch service info again
    if changed:
        service = get_service_by_id(module, auth, service_id)

    service_info = get_service_info(module, auth, service)
    service_info["changed"] = changed

    return service_info


def delete_service(module, auth, service_id):
    service = get_service_by_id(module, auth, service_id)
    if not service:
        return {"changed": False}

    service_info = get_service_info(module, auth, service)

    service_info["changed"] = True

    if module.check_mode:
        return service_info

    try:
        open_url(
            f"{auth.url}/service/{service_id!s}",
            method="DELETE",
            force_basic_auth=True,
            url_username=auth.user,
            url_password=auth.password,
        )
    except Exception as e:
        module.fail_json(msg=f"Service deletion has failed. Error message: {e}")

    return service_info


def get_template_by_name(module, auth, template_name):
    return get_template(module, auth, lambda template: template["NAME"] == template_name)


def get_template_by_id(module, auth, template_id):
    return get_template(module, auth, lambda template: int(template["ID"]) == int(template_id)) if template_id else None


def get_template_id(module, auth, requested_id, requested_name):
    template = (
        get_template_by_id(module, auth, requested_id)
        if requested_id
        else get_template_by_name(module, auth, requested_name)
    )

    if template:
        return template["ID"]

    return None


def get_service_id_by_name(module, auth, service_name):
    service = get_service_by_name(module, auth, service_name)

    if service:
        return service["ID"]

    return None


def get_connection_info(module):
    url = module.params.get("api_url")
    username = module.params.get("api_username")
    password = module.params.get("api_password")

    if not url:
        url = os.environ.get("ONEFLOW_URL")

    if not username:
        username = os.environ.get("ONEFLOW_USERNAME")

    if not password:
        password = os.environ.get("ONEFLOW_PASSWORD")

    if not (url and username and password):
        module.fail_json(
            msg="One or more connection parameters (api_url, api_username, api_password) were not specified"
        )
    return AuthParams(url=url, user=username, password=password)


def main():
    fields = {
        "api_url": {"required": False, "type": "str"},
        "api_username": {"required": False, "type": "str"},
        "api_password": {"required": False, "type": "str", "no_log": True},
        "service_name": {"required": False, "type": "str"},
        "service_id": {"required": False, "type": "int"},
        "template_name": {"required": False, "type": "str"},
        "template_id": {"required": False, "type": "int"},
        "state": {"default": "present", "choices": ["present", "absent"], "type": "str"},
        "mode": {"required": False, "type": "str"},
        "owner_id": {"required": False, "type": "int"},
        "group_id": {"required": False, "type": "int"},
        "unique": {"default": False, "type": "bool"},
        "wait": {"default": False, "type": "bool"},
        "wait_timeout": {"default": 300, "type": "int"},
        "custom_attrs": {"default": {}, "type": "dict"},
        "role": {"required": False, "type": "str"},
        "cardinality": {"required": False, "type": "int"},
        "force": {"default": False, "type": "bool"},
    }

    module = AnsibleModule(
        argument_spec=fields,
        mutually_exclusive=[
            ["template_id", "template_name", "service_id"],
            ["service_id", "service_name"],
            ["template_id", "template_name", "role"],
            ["template_id", "template_name", "cardinality"],
            ["service_id", "custom_attrs"],
        ],
        required_together=[["role", "cardinality"]],
        supports_check_mode=True,
    )

    auth = get_connection_info(module)
    params = module.params
    service_name = params.get("service_name")
    service_id = params.get("service_id")

    requested_template_id = params.get("template_id")
    requested_template_name = params.get("template_name")
    state = params.get("state")
    permissions = params.get("mode")
    owner_id = params.get("owner_id")
    group_id = params.get("group_id")
    unique = params.get("unique")
    wait = params.get("wait")
    wait_timeout = params.get("wait_timeout")
    custom_attrs = params.get("custom_attrs")
    role = params.get("role")
    cardinality = params.get("cardinality")
    force = params.get("force")

    template_id = None

    if requested_template_id or requested_template_name:
        template_id = get_template_id(module, auth, requested_template_id, requested_template_name)
        if not template_id:
            if requested_template_id:
                module.fail_json(msg=f"There is no template with template_id: {requested_template_id!s}")
            elif requested_template_name:
                module.fail_json(msg=f"There is no template with name: {requested_template_name}")

    if unique and not service_name:
        module.fail_json(msg="You cannot use unique without passing service_name!")

    if template_id and state == "absent":
        module.fail_json(msg="State absent is not valid for template")

    if template_id and state == "present":  # Instantiate a service
        result = create_service_and_operation(
            module,
            auth,
            template_id,
            service_name,
            owner_id,
            group_id,
            permissions,
            custom_attrs,
            unique,
            wait,
            wait_timeout,
        )
    else:
        if not (service_id or service_name):
            module.fail_json(msg="To manage the service at least the service id or service name should be specified!")
        if custom_attrs:
            module.fail_json(msg="You can only set custom_attrs when instantiate service!")

        if not service_id:
            service_id = get_service_id_by_name(module, auth, service_name)
        # The task should be failed when we want to manage a non-existent service identified by its name
        if not service_id and state == "present":
            module.fail_json(msg=f"There is no service with name: {service_name}")

        if state == "absent":
            result = delete_service(module, auth, service_id)
        else:
            result = service_operation(
                module, auth, service_id, owner_id, group_id, permissions, role, cardinality, force, wait, wait_timeout
            )

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2025, Dexter Le <dextersydney2001@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = """
module: pacemaker_stonith
short_description: Manage Pacemaker STONITH
author:
  - Dexter Le (@munchtoast)
version_added: 11.3.0
description:
  - This module manages STONITH in a Pacemaker cluster using the Pacemaker CLI.
seealso:
  - name: Pacemaker STONITH documentation
    description: Complete documentation for Pacemaker STONITH.
    link: https://clusterlabs.org/projects/pacemaker/doc/3.0/Pacemaker_Explained/html/resources.html#stonith
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: partial
    details:
      - Only works when check mode is not enabled.
options:
  state:
    description:
      - Indicate desired state for cluster STONITH.
    choices: [present, absent, enabled, disabled]
    default: present
    type: str
  name:
    description:
      - Specify the STONITH name to create.
    required: true
    type: str
  stonith_type:
    description:
      - Specify the STONITH device type.
    type: str
  stonith_options:
    description:
      - Specify the STONITH option to create.
    type: list
    elements: str
    default: []
  stonith_operations:
    description:
      - List of operations to associate with STONITH.
    type: list
    elements: dict
    default: []
    suboptions:
      operation_action:
        description:
          - Operation action to associate with STONITH.
        type: str
      operation_options:
        description:
          - Operation options to associate with action.
        type: list
        elements: str
  stonith_metas:
    description:
      - List of metadata to associate with STONITH.
    type: list
    elements: str
  stonith_argument:
    description:
      - Action to associate with STONITH.
    type: dict
    suboptions:
      argument_action:
        description:
          - Action to apply to STONITH.
        type: str
        choices: [group, before, after]
      argument_options:
        description:
          - Options to associate with STONITH action.
        type: list
        elements: str
  agent_validation:
    description:
      - Enabled agent validation for STONITH creation.
    type: bool
    default: false
  wait:
    description:
      - Timeout period for polling the STONITH creation.
    type: int
    default: 300
"""

EXAMPLES = """
- name: Create virtual-ip STONITH
  community.general.pacemaker_stonith:
    state: present
    name: virtual-stonith
    stonith_type: fence_virt
    stonith_options:
      - "pcmk_host_list=f1"
    stonith_operations:
      - operation_action: monitor
        operation_options:
          - "interval=30s"
"""

RETURN = """
previous_value:
  description: The value of the STONITH before executing the module.
  type: str
  sample: "  * virtual-stonith\t(stonith:fence_virt):\t Started"
  returned: on success
value:
  description: The value of the STONITH after executing the module.
  type: str
  sample: "  * virtual-stonith\t(stonith:fence_virt):\t Started"
  returned: on success
"""

from ansible_collections.community.general.plugins.module_utils._module_helper import StateModuleHelper
from ansible_collections.community.general.plugins.module_utils._pacemaker import pacemaker_runner, wait_for_resource


class PacemakerStonith(StateModuleHelper):
    module = dict(
        argument_spec=dict(
            state=dict(type="str", default="present", choices=["present", "absent", "enabled", "disabled"]),
            name=dict(type="str", required=True),
            stonith_type=dict(type="str"),
            stonith_options=dict(type="list", elements="str", default=[]),
            stonith_operations=dict(
                type="list",
                elements="dict",
                default=[],
                options=dict(
                    operation_action=dict(type="str"),
                    operation_options=dict(type="list", elements="str"),
                ),
            ),
            stonith_metas=dict(type="list", elements="str"),
            stonith_argument=dict(
                type="dict",
                options=dict(
                    argument_action=dict(type="str", choices=["before", "after", "group"]),
                    argument_options=dict(type="list", elements="str"),
                ),
            ),
            agent_validation=dict(type="bool", default=False),
            wait=dict(type="int", default=300),
        ),
        required_if=[("state", "present", ["stonith_type", "stonith_options"])],
        supports_check_mode=True,
    )

    def __init_module__(self):
        self.runner = pacemaker_runner(self.module)
        self.vars.set("previous_value", self._get()["out"])
        self.vars.set("value", self.vars.previous_value, change=True, diff=True)

    def __quit_module__(self):
        self.vars.set("value", self._get()["out"])

    def _process_command_output(self, fail_on_err, ignore_err_msg=""):
        def process(rc, out, err):
            if fail_on_err and rc != 0 and err and ignore_err_msg not in err:
                self.do_raise(f"pcs failed with error (rc={rc}): {err}")
            out = out.rstrip()
            return None if out == "" else out

        return process

    def _get(self):
        with self.runner("cli_action state name") as ctx:
            result = ctx.run(cli_action="stonith", state="status")
            return dict(rc=result[0], out=result[1] if result[1] != "" else None, err=result[2])

    def fmt_stonith_resource(self):
        return dict(resource_name=self.vars.stonith_type)

    # TODO: Pluralize operation_options in separate PR and remove this helper fmt function
    def fmt_stonith_operations(self):
        modified_stonith_operations = []
        for stonith_operation in self.vars.stonith_operations:
            modified_stonith_operations.append(
                dict(
                    operation_action=stonith_operation.get("operation_action"),
                    operation_option=stonith_operation.get("operation_options"),
                )
            )
        return modified_stonith_operations

    def state_absent(self):
        with self.runner(
            "cli_action state name",
            output_process=self._process_command_output(True, "does not exist"),
            check_mode_skip=True,
        ) as ctx:
            ctx.run(cli_action="stonith")

    def state_present(self):
        with self.runner(
            "cli_action state name resource_type resource_option resource_operation resource_meta resource_argument agent_validation",
            output_process=self._process_command_output(True, "already exists"),
            check_mode_skip=True,
        ) as ctx:
            ctx.run(
                cli_action="stonith",
                resource_type=self.fmt_stonith_resource(),
                resource_option=self.vars.stonith_options,
                resource_operation=self.fmt_stonith_operations(),
                resource_meta=self.vars.stonith_metas,
                resource_argument=self.vars.stonith_argument,
            )
        if not self.module.check_mode and self.vars.wait:
            wait_for_resource(self.runner, "stonith", self.vars.name, self.vars.wait)

    def state_enabled(self):
        with self.runner(
            "cli_action state name", output_process=self._process_command_output(True, "Starting"), check_mode_skip=True
        ) as ctx:
            ctx.run(cli_action="stonith")

    def state_disabled(self):
        with self.runner(
            "cli_action state name", output_process=self._process_command_output(True, "Stopped"), check_mode_skip=True
        ) as ctx:
            ctx.run(cli_action="stonith")


def main():
    PacemakerStonith.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_sm_admins_roles_info
short_description: Information module for organizations _sm _admins _roles
description:
  - Information module for Organizations Sm Admins Roles Info.
  - Get all organizations _sm _admins _roles.
  - List the Limited Access Roles for an organization.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - Information module for Organizations Sm Admins Roles Info.
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - Information module for Organizations Sm Admins Roles Info.
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 50.
    type: int
  startingAfter:
    description:
      - Information module for Organizations Sm Admins Roles Info.
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - Information module for Organizations Sm Admins Roles Info.
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for sm getOrganizationSmAdminsRoles
    description: Complete reference of the getOrganizationSmAdminsRoles API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-sm-admins-roles
notes:
  - SDK Method used are
    sm.Sm.get_organization_sm_admins_roles,
  - Paths used are
    get /organizations/{organizationId}/sm/admins/roles,
"""

EXAMPLES = r"""
- name: Get all organizations _sm _admins _roles
  cisco.meraki.organizations_sm_admins_roles_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {
      "items": [
        {
          "roleId": "string",
          "name": "string",
          "scope": "string",
          "tags": [
            "string"
          ]
        }
      ],
      "meta": {
        "counts": {
          "items": {
            "total": 0,
            "remaining": 0
          }
        }
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: organizations_firmware_upgrades_by_device_info
short_description: Information module for organizations _firmware _upgrades _bydevice
description:
  - Information module for Organizations Firmware Upgrades Bydevice Info.
  - Get all organizations _firmware _upgrades _bydevice.
  - >
    Get firmware upgrade status for the filtered devices. This endpoint currently
    only supports Meraki switches and access points.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module_info
  - cisco.meraki.module_info_pagination
author: Francisco Munoz (@fmunoz)
options:
  headers:
    description: Additional headers.
    type: dict
  organizationId:
    description:
      - Information module for Organizations Firmware Upgrades Bydevice Info.
      - OrganizationId path parameter. Organization ID.
    type: str
  perPage:
    description:
      - Information module for Organizations Firmware Upgrades Bydevice Info.
      - PerPage query parameter. The number of entries per page returned. Acceptable
        range is 3 - 1000. Default is 50.
    type: int
  startingAfter:
    description:
      - Information module for Organizations Firmware Upgrades Bydevice Info.
      - >
        StartingAfter query parameter. A token used by the server to indicate the
        start of the page. Often this is a timestamp or an ID but it is not limited
        to those. This parameter should not be defined by client applications. The
        link for the first, last, prev, or next page in the HTTP Link header should
        define it.
    type: str
  endingBefore:
    description:
      - Information module for Organizations Firmware Upgrades Bydevice Info.
      - >
        EndingBefore query parameter. A token used by the server to indicate the end
        of the page. Often this is a timestamp or an ID but it is not limited to those.
        This parameter should not be defined by client applications. The link for
        the first, last, prev, or next page in the HTTP Link header should define
        it.
    type: str
  networkIds:
    description:
      - Information module for Organizations Firmware Upgrades Bydevice Info.
      - NetworkIds query parameter. Optional parameter to filter by network.
    elements: str
    type: list
  serials:
    description:
      - Information module for Organizations Firmware Upgrades Bydevice Info.
      - >
        Serials query parameter. Optional parameter to filter by serial number. All
        returned devices will have a serial number that is an exact match.
    elements: str
    type: list
  macs:
    description:
      - Information module for Organizations Firmware Upgrades Bydevice Info.
      - >
        Macs query parameter. Optional parameter to filter by one or more MAC addresses
        belonging to devices. All devices returned belong to MAC addresses that are
        an exact match.
    elements: str
    type: list
  firmwareUpgradeBatchIds:
    description:
      - Information module for Organizations Firmware Upgrades Bydevice Info.
      - FirmwareUpgradeBatchIds query parameter. Optional parameter to filter by firmware
        upgrade batch ids.
    elements: str
    type: list
  upgradeStatuses:
    description:
      - Information module for Organizations Firmware Upgrades Bydevice Info.
      - UpgradeStatuses query parameter. Optional parameter to filter by firmware
        upgrade statuses.
    elements: str
    type: list
  currentUpgradesOnly:
    description:
      - Information module for Organizations Firmware Upgrades Bydevice Info.
      - CurrentUpgradesOnly query parameter. Optional parameter to filter to only
        current or pending upgrade statuses.
    type: bool
  limitPerDevice:
    description:
      - Information module for Organizations Firmware Upgrades Bydevice Info.
      - >
        LimitPerDevice query parameter. Optional parameter to limit the number of
        upgrade statuses returned per device. If omitted, a value of 5 is used.
    type: int
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for organizations getOrganizationFirmwareUpgradesByDevice
    description: Complete reference of the getOrganizationFirmwareUpgradesByDevice
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!get-organization-firmware-upgrades-by-device
notes:
  - SDK Method used are
    organizations.Organizations.get_organization_firmware_upgrades_by_device,
  - Paths used are
    get /organizations/{organizationId}/firmware/upgrades/byDevice,
"""

EXAMPLES = r"""
- name: Get all organizations _firmware _upgrades _bydevice
  cisco.meraki.organizations_firmware_upgrades_by_device_info:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    perPage: 0
    startingAfter: string
    endingBefore: string
    networkIds: []
    serials: []
    macs: []
    firmwareUpgradeBatchIds: []
    upgradeStatuses: []
    currentUpgradesOnly: true
    limitPerDevice: 0
    organizationId: string
    total_pages: -1
    direction: next
  register: result
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: list
  elements: dict
  sample: >
    [
      {
        "serial": "string",
        "name": "string",
        "deviceStatus": "string",
        "checkinFinishedAt": {},
        "checkinStartedAt": {},
        "detailedStatus": {},
        "downloadFinishedAt": {},
        "downloadStartedAt": {},
        "downloadStatus": {},
        "installFinishedAt": {},
        "installStartedAt": {},
        "installStatus": {},
        "verifyFinishedAt": {},
        "verifyStartedAt": {},
        "verifyStatus": {},
        "upgrade": {
          "time": "string",
          "fromVersion": {
            "id": "string",
            "shortName": "string",
            "releaseDate": "string"
          },
          "toVersion": {
            "id": "string",
            "shortName": "string",
            "releaseDate": "string"
          },
          "status": "string",
          "id": "string",
          "upgradeBatchId": "string",
          "staged": {
            "group": {
              "id": "string"
            }
          }
        }
      }
    ]
"""
